@@ -16,12 +16,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-
-message ()
-{
+message () {
   echo "USAGE: $(basename "$0") [-d datadir] [-f format] [-u] <direction> [in [out]]"
   echo " -d datadir       directory of linguistic data"
-  echo " -f format        one of: txt (default), html, rtf, odt, docx, wxml, xlsx, pptx,"
+  echo " -f format        one of: txt (default), html, rtf, odt, odp, docx, wxml, xlsx, pptx,"
   echo "                  xpresstag, html-noent, html-alt, latex, latex-raw, line"
   echo " -a               display ambiguity"
   echo " -u               don't display marks '*' for unknown words"
@@ -30,21 +28,21 @@ message ()
   echo " -o direction     translation direction using the translation memory,"
   echo "                  by default 'direction' is used instead"
   echo " -l               lists the available translation directions and exits"
+  # This is wrong and should be deprecated, but until all tools consider \0 flush without -z then this helps to have
+  echo " -z               force null-flush mode on all parts of the pipe"
   echo " direction        typically, LANG1-LANG2, but see modes.xml in language data"
   echo " in               input file (stdin by default)"
   echo " out              output file (stdout by default)"
   exit 1
 }
 
-list_directions ()
-{
+list_directions () {
   for mode in "$DATADIR"/modes/*.mode; do
     echo "  $(basename "${mode%%.mode}")"
   done
 }
 
-locale_utf8 ()
-{
+locale_utf8 () {
   export LC_CTYPE
   LC_CTYPE=$(locale -a|grep -i "utf[.-]*8"|head -1)
   if [[ -z ${LC_CTYPE} ]]; then
@@ -63,8 +61,24 @@ check_encoding () {
   fi
 }
 
-test_zip ()
-{
+check_transfuse () {
+  local has_tf
+  has_tf=$(command -v tf-extract)
+  if [[ "$APERTIUM_TRANSFUSE" = "1" || "$APERTIUM_TRANSFUSE" = "yes" ]]; then
+    if [[ -z "$has_tf" ]]; then
+      echo "APERTIUM_TRANSFUSE=$APERTIUM_TRANSFUSE but couldn't find Transfuse's tf-extract in PATH"
+      exit 1
+    fi
+  fi
+
+  if [[ "$APERTIUM_TRANSFUSE" = "0" || "$APERTIUM_TRANSFUSE" = "no" || -z "$has_tf" ]]; then
+    USE_TRANSFUSE=no
+  elif [[ "$APERTIUM_TRANSFUSE" = "1" || "$APERTIUM_TRANSFUSE" = "yes" || -n "$has_tf" ]]; then
+    USE_TRANSFUSE=yes
+  fi
+}
+
+test_zip () {
   if ! command -v zip &>/dev/null; then
     echo "Error: Install 'zip' command in your system";
     exit 1;
@@ -76,16 +90,14 @@ test_zip ()
   fi
 }
 
-test_gawk ()
-{
+test_gawk () {
   if ! command -v gawk &>/dev/null; then
     echo "Error: Install 'gawk' in your system"
     exit 1
   fi
 }
 
-translate_latex()
-{
+translate_latex() {
   test_gawk
 
   if [[ -z "$INFILE" || "$INFILE" = /dev/stdin ]]; then
@@ -94,25 +106,21 @@ translate_latex()
     BORRAFICHERO="true"
   fi
   check_encoding "${INFILE}"
-  locale_utf8
 
   set -o pipefail
-  "$APERTIUM_PATH/apertium-prelatex" "$INFILE" | \
-    "$APERTIUM_PATH/apertium-utils-fixlatex" | \
-    "$APERTIUM_PATH/apertium-deslatex" "${FORMAT_OPTIONS[@]}" | \
-    if [ "$TRANSLATION_MEMORY_FILE" = "" ];
+  apertium-prelatex "$INFILE" | \
+    apertium-utils-fixlatex | \
+    apertium-deslatex "${FORMAT_OPTIONS[@]}" | \
+    if [[ -z "$TRANSLATION_MEMORY_FILE" ]];
     then cat;
-    else "$APERTIUM_PATH/lt-tmxproc" "$TMCOMPFILE";
+    else lt-tmxproc "$TMCOMPFILE";
     fi | \
-      if [ ! -x "$DATADIR/modes/$PAIR.mode" ]; then
-      sh "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-    else "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-    fi | \
-      "$APERTIUM_PATH/apertium-relatex"| \
+      bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | \
+      apertium-relatex| \
       awk '{gsub("</CONTENTS-noeos>", "</CONTENTS>"); print;}' | \
-      if [ "$REDIR" == "" ]; then "$APERTIUM_PATH/apertium-postlatex-raw"; else "$APERTIUM_PATH/apertium-postlatex-raw" > "$SALIDA"; fi
+      if [[ -z "$REDIR" ]]; then apertium-postlatex-raw; else apertium-postlatex-raw > "$OUT_FILE"; fi
 
-    if [ "$BORRAFICHERO" = "true" ]; then
+    if [[ "$BORRAFICHERO" = "true" ]]; then
       rm -Rf "$INFILE"
     fi
 }
@@ -120,70 +128,74 @@ translate_latex()
 # TODO: What was the intended difference between this and
 # translate_latex? They were identical, apart from one not cleaning up
 # tmp files
-translate_latex_raw()
-{
+translate_latex_raw() {
   translate_latex
 }
 
 
-translate_odt ()
-{
+translate_odt () {
+  check_transfuse
+  if [[ $USE_TRANSFUSE = "yes" ]]; then
+    tf-extract -f odt "$INFILE" | bash <(apertium-wblank-mode -z "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+    return
+  fi
+
   INPUT_TMPDIR=$(mktemp -d "$TMPDIR/apertium.XXXXXXXX")
 
-  locale_utf8
   test_zip
 
-  if [ "$INFILE" = "" ]; then
+  if [[ -z "$INFILE" ]]; then
     INFILE=$(mktemp "$TMPDIR/apertium.XXXXXXXX")
     cat > "$INFILE"
     BORRAFICHERO="true"
   fi
-  OTRASALIDA=$(mktemp "$TMPDIR/apertium.XXXXXXXX")
+  TMP_OUT_FILE=$(mktemp "$TMPDIR/apertium.XXXXXXXX")
 
   unzip -q -o -d "$INPUT_TMPDIR" "$INFILE"
   find "$INPUT_TMPDIR" -name content.xml -o -name styles.xml |\
   awk '{printf "<file name=\"" $0 "\"/>"; PART = $0; while(getline < PART) printf(" %s", $0); printf("\n");}' |\
-  "$APERTIUM_PATH/apertium-desodt" "${FORMAT_OPTIONS[@]}" |\
-  if [ "$TRANSLATION_MEMORY_FILE" = "" ];
+  apertium-desodt "${FORMAT_OPTIONS[@]}" |\
+  if [[ -z "$TRANSLATION_MEMORY_FILE" ]];
   then cat;
-  else "$APERTIUM_PATH/lt-tmxproc" "$TMCOMPFILE";
+  else lt-tmxproc "$TMCOMPFILE";
   fi | \
-    if [ ! -x "$DATADIR/modes/$PAIR.mode" ]; then
-    sh "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-  else "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-  fi | \
-    "$APERTIUM_PATH/apertium-reodt"|\
+    bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | \
+    apertium-reodt|\
   awk '{punto = index($0, "/>") + 3; cabeza = substr($0, 1, punto-1); cola = substr($0, punto); n1 = substr(cabeza, index(cabeza, "\"")+1); name = substr(n1, 1, index(n1, "\"")-1); gsub("[?]> ", "?>\n", cola); print cola > name;}'
   VUELVE=$(pwd)
   cd "$INPUT_TMPDIR"
   rm -Rf ObjectReplacements
-  zip -q -r - . >"$OTRASALIDA"
+  zip -q -r - . >"$TMP_OUT_FILE"
   cd "$VUELVE"
   rm -Rf "$INPUT_TMPDIR"
 
-  if [ "$BORRAFICHERO" = "true" ]; then
+  if [[ "$BORRAFICHERO" = "true" ]]; then
     rm -Rf "$INFILE";
   fi
 
-  if [ "$REDIR" == "" ]; then cat "$OTRASALIDA"; else cat "$OTRASALIDA" > "$SALIDA"; fi
-  rm -Rf "$OTRASALIDA"
+  if [[ -z "$REDIR" ]]; then cat "$TMP_OUT_FILE"; else cat "$TMP_OUT_FILE" > "$OUT_FILE"; fi
+  rm -Rf "$TMP_OUT_FILE"
 }
 
-translate_docx ()
-{
+translate_docx () {
+  check_transfuse
+  if [[ $USE_TRANSFUSE = "yes" ]]; then
+    tf-extract -f docx "$INFILE" | bash <(apertium-wblank-mode -z "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+    return
+  fi
+
   INPUT_TMPDIR=$(mktemp -d "$TMPDIR/apertium.XXXXXXXX")
 
-  locale_utf8
   test_zip
 
-  if [ "$INFILE" = "" ]; then
+  if [[ -z "$INFILE" ]]; then
     INFILE=$(mktemp "$TMPDIR/apertium.XXXXXXXX")
     cat > "$INFILE"
     BORRAFICHERO="true"
   fi
-  OTRASALIDA=$(mktemp "$TMPDIR/apertium.XXXXXXXX")
+  TMP_OUT_FILE=$(mktemp "$TMPDIR/apertium.XXXXXXXX")
 
-  if [ "$UWORDS" = "no" ]; then
+  if [[ "$UWORDS" = "no" ]]; then
     OPCIONU="-u";
   else OPCIONU="";
   fi
@@ -192,54 +204,55 @@ translate_docx ()
 
   find "$INPUT_TMPDIR" -name "*.xlsx" -print0 | while read -r -d '' i; do
     LOCALTEMP=$(mktemp "$TMPDIR/apertium.XXXXXXXX");
-    "$APERTIUM_PATH/apertium" -f xlsx -d "$DATADIR" "$OPCIONU" "$PAIR" <"$i" >"$LOCALTEMP";
+    apertium -f xlsx -d "$DATADIR" "$OPCIONU" "$PAIR" <"$i" >"$LOCALTEMP";
     cp "$LOCALTEMP" "$i";
     rm "$LOCALTEMP";
   done;
 
   find "$INPUT_TMPDIR" -name "*.xml" |\
-  grep -E -v -i '(settings|theme|styles|font|rels|docProps)' |\
-  awk '{printf "<file name=\"" $0 "\"/>"; PART = $0; while(getline < PART) printf(" %s", $0); printf("\n");}' |\
-  "$APERTIUM_PATH/apertium-deswxml" "${FORMAT_OPTIONS[@]}" |\
-  if [ "$TRANSLATION_MEMORY_FILE" = "" ];
+  grep -E -v -i '(settings|theme|styles|font|rels|docProps|Content_Types)' |\
+  apertium-adapt-docx -n |\
+  apertium-deswxml "${FORMAT_OPTIONS[@]}" |\
+  if [[ -z "$TRANSLATION_MEMORY_FILE" ]];
   then cat;
-  else "$APERTIUM_PATH/lt-tmxproc" "$TMCOMPFILE";
+  else lt-tmxproc "$TMCOMPFILE";
   fi | \
-    if [ ! -x "$DATADIR/modes/$PAIR.mode" ]; then
-    sh "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-  else "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-  fi | \
-    "$APERTIUM_PATH/apertium-rewxml"|\
+    bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | \
+    apertium-rewxml|\
   awk '{punto = index($0, "/>") + 3; cabeza = substr($0, 1, punto-1); cola = substr($0, punto); n1 = substr(cabeza, index(cabeza, "\"")+1); name = substr(n1, 1, index(n1, "\"")-1); gsub("[?]> ", "?>\n", cola); print cola > name;}'
   VUELVE=$(pwd)
   cd "$INPUT_TMPDIR"
-  zip -q -r - . >"$OTRASALIDA"
+  zip -q -r - . >"$TMP_OUT_FILE"
   cd "$VUELVE"
   rm -Rf "$INPUT_TMPDIR"
 
-  if [ "$BORRAFICHERO" = "true" ]; then
+  if [[ "$BORRAFICHERO" = "true" ]]; then
     rm -Rf "$INFILE";
   fi
 
-  if [ "$REDIR" == "" ]; then cat "$OTRASALIDA"; else cat "$OTRASALIDA" > "$SALIDA"; fi
-  rm -Rf "$OTRASALIDA"
+  if [[ -z "$REDIR" ]]; then cat "$TMP_OUT_FILE"; else cat "$TMP_OUT_FILE" > "$OUT_FILE"; fi
+  rm -Rf "$TMP_OUT_FILE"
 }
 
-translate_pptx ()
-{
+translate_pptx () {
+  check_transfuse
+  if [[ $USE_TRANSFUSE = "yes" ]]; then
+    tf-extract -f pptx "$INFILE" | bash <(apertium-wblank-mode -z "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+    return
+  fi
+
   INPUT_TMPDIR=$(mktemp -d "$TMPDIR/apertium.XXXXXXXX")
 
-  locale_utf8
   test_zip
 
-  if [ "$INFILE" = "" ]; then
+  if [[ -z "$INFILE" ]]; then
     INFILE=$(mktemp "$TMPDIR/apertium.XXXXXXXX")
     cat > "$INFILE"
     BORRAFICHERO="true"
   fi
-  OTRASALIDA=$(mktemp "$TMPDIR/apertium.XXXXXXXX")
+  TMP_OUT_FILE=$(mktemp "$TMPDIR/apertium.XXXXXXXX")
 
-  if [ "$UWORDS" = "no" ]; then
+  if [[ "$UWORDS" = "no" ]]; then
     OPCIONU="-u";
   else OPCIONU="";
   fi
@@ -248,138 +261,130 @@ translate_pptx ()
 
   find "$INPUT_TMPDIR" -name "*.xlsx" -print0 | while read -r -d '' i; do
     LOCALTEMP=$(mktemp "$TMPDIR/apertium.XXXXXXXX")
-    "$APERTIUM_PATH/apertium" -f xlsx -d "$DATADIR" "$OPCIONU" "$PAIR" <"$i" >"$LOCALTEMP";
+    apertium -f xlsx -d "$DATADIR" "$OPCIONU" "$PAIR" <"$i" >"$LOCALTEMP";
     cp "$LOCALTEMP" "$i"
     rm "$LOCALTEMP"
   done;
 
-  find . -path '**/slides/slide*.xml' |\
+  find "$INPUT_TMPDIR" -path '**/slides/slide*.xml' |\
   awk '{printf "<file name=\"" $0 "\"/>"; PART = $0; while(getline < PART) printf(" %s", $0); printf("\n");}' |\
-  "$APERTIUM_PATH/apertium-despptx" "${FORMAT_OPTIONS[@]}" |\
-  if [ "$TRANSLATION_MEMORY_FILE" = "" ];
+  apertium-despptx "${FORMAT_OPTIONS[@]}" |\
+  if [[ -z "$TRANSLATION_MEMORY_FILE" ]];
   then cat;
-  else "$APERTIUM_PATH/lt-tmxproc" "$TMCOMPFILE";
+  else lt-tmxproc "$TMCOMPFILE";
   fi | \
-    if [ ! -x "$DATADIR/modes/$PAIR.mode" ]; then
-    sh "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-  else "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-  fi | \
-    "$APERTIUM_PATH/apertium-repptx" |\
+    bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | \
+    apertium-repptx |\
   awk '{punto = index($0, "/>") + 3; cabeza = substr($0, 1, punto-1); cola = substr($0, punto); n1 = substr(cabeza, index(cabeza, "\"")+1); name = substr(n1, 1, index(n1, "\"")-1); gsub("[?]> ", "?>\n", cola); print cola > name;}'
   VUELVE=$(pwd)
   cd "$INPUT_TMPDIR"
-  zip -q -r - . >"$OTRASALIDA"
+  zip -q -r - . >"$TMP_OUT_FILE"
   cd "$VUELVE"
   rm -Rf "$INPUT_TMPDIR"
 
-  if [ "$BORRAFICHERO" = "true" ]; then
+  if [[ "$BORRAFICHERO" = "true" ]]; then
     rm -Rf "$INFILE";
   fi
 
-  if [ "$REDIR" == "" ]; then cat "$OTRASALIDA"; else cat "$OTRASALIDA" > "$SALIDA"; fi
-  rm -Rf "$OTRASALIDA"
+  if [[ -z "$REDIR" ]]; then cat "$TMP_OUT_FILE"; else cat "$TMP_OUT_FILE" > "$OUT_FILE"; fi
+  rm -Rf "$TMP_OUT_FILE"
 }
 
 
-translate_xlsx ()
-{
+translate_xlsx () {
   INPUT_TMPDIR=$(mktemp -d "$TMPDIR/apertium.XXXXXXXX")
 
-  locale_utf8
   test_zip
 
-  if [ "$INFILE" = "" ]; then
+  if [[ -z "$INFILE" ]]; then
     INFILE=$(mktemp "$TMPDIR/apertium.XXXXXXXX")
     cat > "$INFILE"
     BORRAFICHERO="true"
   fi
-  OTRASALIDA=$(mktemp "$TMPDIR/apertium.XXXXXXXX")
+  TMP_OUT_FILE=$(mktemp "$TMPDIR/apertium.XXXXXXXX")
 
   unzip -q -o -d "$INPUT_TMPDIR" "$INFILE"
   find "$INPUT_TMPDIR" -name "sharedStrings.xml" |\
   awk '{printf "<file name=\"" $0 "\"/>"; PART = $0; while(getline < PART) printf(" %s", $0); printf("\n");}' |\
-  "$APERTIUM_PATH/apertium-desxlsx" "${FORMAT_OPTIONS[@]}" |\
-  if [ "$TRANSLATION_MEMORY_FILE" = "" ];
+  apertium-desxlsx "${FORMAT_OPTIONS[@]}" |\
+  if [[ -z "$TRANSLATION_MEMORY_FILE" ]];
   then cat;
-  else "$APERTIUM_PATH/lt-tmxproc" "$TMCOMPFILE";
+  else lt-tmxproc "$TMCOMPFILE";
   fi | \
-    if [ ! -x "$DATADIR/modes/$PAIR.mode" ]; then
-    sh "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-  else "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-  fi | \
-    "$APERTIUM_PATH/apertium-rexlsx" |\
+    bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | \
+    apertium-rexlsx |\
   awk '{punto = index($0, "/>") + 3; cabeza = substr($0, 1, punto-1); cola = substr($0, punto); n1 = substr(cabeza, index(cabeza, "\"")+1); name = substr(n1, 1, index(n1, "\"")-1); gsub("[?]> ", "?>\n", cola); print cola > name;}'
   VUELVE=$(pwd)
   cd "$INPUT_TMPDIR"
-  zip -q -r - . >"$OTRASALIDA"
+  zip -q -r - . >"$TMP_OUT_FILE"
   cd "$VUELVE"
   rm -Rf "$INPUT_TMPDIR"
 
-  if [ "$BORRAFICHERO" = "true" ]; then
+  if [[ "$BORRAFICHERO" = "true" ]]; then
     rm -Rf "$INFILE";
   fi
 
-  if [ "$REDIR" == "" ]; then cat "$OTRASALIDA"; else cat "$OTRASALIDA" > "$SALIDA"; fi
-  rm -Rf "$OTRASALIDA"
+  if [[ -z "$REDIR" ]]; then cat "$TMP_OUT_FILE"; else cat "$TMP_OUT_FILE" > "$OUT_FILE"; fi
+  rm -Rf "$TMP_OUT_FILE"
 }
 
-translate_htmlnoent ()
-{
-  "$APERTIUM_PATH/apertium-deshtml" "${FORMAT_OPTIONS[@]}" "$INFILE" | \
-      if [ "$TRANSLATION_MEMORY_FILE" = "" ]; then
-          cat
-      else "$APERTIUM_PATH/lt-tmxproc" "$TMCOMPFILE";
-      fi | if [ ! -x "$DATADIR/modes/$PAIR.mode" ]; then
-      sh "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-  else "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-  fi | if [ "$FORMAT" = "none" ]; then
-      if [ "$REDIR" == "" ]; then cat; else cat > "$SALIDA"; fi
-  else
-    if [ "$REDIR" == "" ]; then "$APERTIUM_PATH/apertium-rehtml-noent"; else "$APERTIUM_PATH/apertium-rehtml-noent" > "$SALIDA"; fi
+translate_html () {
+  check_transfuse
+  if [[ $USE_TRANSFUSE = "yes" ]]; then
+    tf-extract -f html "$INFILE" | bash <(apertium-wblank-mode -z "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+    return
   fi
+
+  apertium-deshtml "${FORMAT_OPTIONS[@]}" "$INFILE" | \
+    if [[ -z "$TRANSLATION_MEMORY_FILE" ]]; then
+      cat
+    else lt-tmxproc "$TMCOMPFILE";
+    fi | bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then apertium-rehtml; else apertium-rehtml > "$OUT_FILE"; fi
 }
 
-translate_htmlalt ()
-{
-  "$APERTIUM_PATH/apertium-deshtml-alt" "${FORMAT_OPTIONS[@]}" "$INFILE" | \
-      if [ "$TRANSLATION_MEMORY_FILE" = "" ]; then
-          cat
-      else "$APERTIUM_PATH/lt-tmxproc" "$TMCOMPFILE";
-      fi | if [ ! -x "$DATADIR/modes/$PAIR.mode" ]; then
-      sh "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-  else "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-  fi | if [ "$FORMAT" = "none" ]; then
-      if [ "$REDIR" == "" ]; then cat; else cat > "$SALIDA"; fi
-  else
-    if [ "$REDIR" == "" ]; then "$APERTIUM_PATH/apertium-rehtml-noent"; else "$APERTIUM_PATH/apertium-rehtml-noent" > "$SALIDA"; fi
+translate_htmlnoent () {
+  check_transfuse
+  if [[ $USE_TRANSFUSE = "yes" ]]; then
+    tf-extract -f html "$INFILE" | bash <(apertium-wblank-mode -z "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+    return
   fi
+
+  apertium-deshtml "${FORMAT_OPTIONS[@]}" "$INFILE" | \
+    if [[ -z "$TRANSLATION_MEMORY_FILE" ]]; then
+      cat
+    else lt-tmxproc "$TMCOMPFILE";
+    fi | bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then apertium-rehtml-noent; else apertium-rehtml-noent > "$OUT_FILE"; fi
 }
 
-translate_line ()
-{
+translate_htmlalt () {
+  apertium-deshtml-alt "${FORMAT_OPTIONS[@]}" "$INFILE" | \
+    if [[ -z "$TRANSLATION_MEMORY_FILE" ]]; then
+      cat
+    else lt-tmxproc "$TMCOMPFILE";
+    fi | bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then apertium-rehtml-noent; else apertium-rehtml-noent > "$OUT_FILE"; fi
+}
+
+translate_line () {
   # TODO: lt-proc inserts spaces before parts of mwe's that cross
   # lines, even though the parts get output as individual lu's
-  "$APERTIUM_PATH/apertium-destxt" -n "$INFILE" | \
+  apertium-destxt -n "$INFILE" | \
       sed 's/[[:space:]]*\[$/[][/' |\
-      if [ "$TRANSLATION_MEMORY_FILE" = "" ]; then
+      if [[ -z "$TRANSLATION_MEMORY_FILE" ]]; then
           cat
       else
-        "$APERTIUM_PATH/lt-tmxproc" "$TMCOMPFILE";
+        lt-tmxproc "$TMCOMPFILE";
       fi | \
-          if [ ! -x "$DATADIR/modes/$PAIR.mode" ]; then
-              sh "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-          else
-            "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-          fi | \
-              if [ "$REDIR" == "" ]; then
-                  "$APERTIUM_PATH/apertium-retxt"
+          bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | \
+              if [[ -z "$REDIR" ]]; then
+                  apertium-retxt
               else
-                  "$APERTIUM_PATH/apertium-retxt" > "$SALIDA"
+                  apertium-retxt > "$OUT_FILE"
               fi
 }
 
 
-
+# Require UTF-8 locale
+locale_utf8
 
 ##########################################################
 # Option and argument parsing, setting globals variables #
@@ -393,10 +398,11 @@ trap 'rm -Rf "$TMCOMPFILE"' EXIT
 PAIR=""
 INFILE="/dev/stdin"
 FORMAT="txt"
-DATADIR=$DEFAULT_DIRECTORY
+DATADIR=$APERTIUM_DATADIR
 TRANSLATION_MEMORY_DIRECTION=$PAIR
 LIST_MODES_AND_EXIT=false
 FORMAT_OPTIONS=()
+NULL_FLUSH=()
 
 # Skip (but store) non-option arguments that come before options:
 declare -a ARGS_PREOPT ARGS_ALL
@@ -411,7 +417,7 @@ while [[ $OPTIND -le $# ]]; do
 done
 
 
-while getopts ":uahlf:d:m:o:n" opt; do
+while getopts ":uahlzf:d:m:o:n" opt; do
   case "$opt" in
     f) FORMAT=$OPTARG ;;
     d) DATADIR=$OPTARG ;;
@@ -421,6 +427,7 @@ while getopts ":uahlf:d:m:o:n" opt; do
     n) FORMAT_OPTIONS+=(-n) ;;
     a) OPTION_TAGGER="-m" ;;
     l) LIST_MODES_AND_EXIT=true ;;
+    z) NULL_FLUSH+=(-z) ;;
     h) message ;;
     \?) echo "ERROR: Unknown option $OPTARG" >&2; message >&2 ;;
     :) echo "ERROR: $OPTARG requires an argument" >&2; message >&2 ;;
@@ -435,7 +442,7 @@ set -- "${ARGS_PREOPT[@]}" "$@"
 
 case "$#" in
   3)
-    SALIDA=$3
+    OUT_FILE=$3
     REDIR=">"
     INFILE=$2
     PAIR=$1
@@ -461,7 +468,7 @@ elif [[ ! -r "$INFILE" ]]; then
 fi
 
 if [[ -n $TRANSLATION_MEMORY_FILE ]]; then
-  if ! "$APERTIUM_PATH/lt-tmxcomp" "$TRANSLATION_MEMORY_DIRECTION" "$TRANSLATION_MEMORY_FILE" "$TMCOMPFILE" >/dev/null; then
+  if ! lt-tmxcomp "$TRANSLATION_MEMORY_DIRECTION" "$TRANSLATION_MEMORY_FILE" "$TMCOMPFILE" >/dev/null; then
     echo "Error: Cannot compile TM '$TRANSLATION_MEMORY_FILE'" >&2
     echo"   hint: use -o parameter" >&2
     message >&2
@@ -476,7 +483,7 @@ fi
 if [[ ! -e "$DATADIR/modes/$PAIR.mode" ]]; then
   echo -n "Error: Mode $PAIR does not exist"
   c=$(find "$DATADIR/modes" -name '*.mode' | wc -l)
-  if [ "$c" -le 1 ]; then
+  if [[ "$c" -le 1 ]]; then
     echo "."
   else
     echo ". Try one of:"
@@ -489,39 +496,39 @@ fi
 
 case "$FORMAT" in
   none)
-    if [ "$UWORDS" = "no" ]; then
+    if [[ "$UWORDS" = "no" ]]; then
       OPTION="-n";
     else OPTION="-g";
     fi
     ;;
-  txt|rtf|html|xpresstag|mediawiki)
-    if [ "$UWORDS" = "no" ]; then OPTION="-n";
+  txt|rtf|xpresstag|mediawiki)
+    if [[ "$UWORDS" = "no" ]]; then OPTION="-n";
     else OPTION="-g";
     fi;
     ;;
-  odt)
-    if [ "$UWORDS" = "no" ]; then OPTION="-n";
+  odt|odp)
+    if [[ "$UWORDS" = "no" ]]; then OPTION="-n";
     else OPTION="-g";
     fi;
     translate_odt
     exit 0
     ;;
   latex)
-    if [ "$UWORDS" = "no" ]; then OPTION="-n";
+    if [[ "$UWORDS" = "no" ]]; then OPTION="-n";
     else OPTION="-g";
     fi;
     translate_latex
     exit 0
     ;;
   latex-raw)
-    if [ "$UWORDS" = "no" ]; then OPTION="-n";
+    if [[ "$UWORDS" = "no" ]]; then OPTION="-n";
     else OPTION="-g";
     fi;
     translate_latex_raw
     exit 0
     ;;
   line)
-    if [ "$UWORDS" = "no" ]; then OPTION="-n";
+    if [[ "$UWORDS" = "no" ]]; then OPTION="-n";
     else OPTION="-g";
     fi;
     translate_line
@@ -530,35 +537,42 @@ case "$FORMAT" in
 
 
   docx)
-    if [ "$UWORDS" = "no" ]; then OPTION="-n";
+    if [[ "$UWORDS" = "no" ]]; then OPTION="-n";
     else OPTION="-g";
     fi;
     translate_docx
     exit 0
     ;;
   xlsx)
-    if [ "$UWORDS" = "no" ]; then OPTION="-n";
+    if [[ "$UWORDS" = "no" ]]; then OPTION="-n";
     else OPTION="-g";
     fi;
     translate_xlsx
     exit 0
     ;;
   pptx)
-    if [ "$UWORDS" = "no" ]; then OPTION="-n";
+    if [[ "$UWORDS" = "no" ]]; then OPTION="-n";
     else OPTION="-g";
     fi;
     translate_pptx
     exit 0
     ;;
+  html)
+    if [[ "$UWORDS" = "no" ]]; then OPTION="-n";
+    else OPTION="-g";
+    fi;
+    translate_html
+    exit 0
+    ;;
   html-noent)
-    if [ "$UWORDS" = "no" ]; then OPTION="-n";
+    if [[ "$UWORDS" = "no" ]]; then OPTION="-n";
     else OPTION="-g";
     fi;
     translate_htmlnoent
     exit 0
     ;;
   html-alt)
-    if [ "$UWORDS" = "no" ]; then OPTION="-n";
+    if [[ "$UWORDS" = "no" ]]; then OPTION="-n";
     else OPTION="-g";
     fi;
     translate_htmlalt
@@ -566,10 +580,9 @@ case "$FORMAT" in
     ;;
 
   wxml)
-    if [ "$UWORDS" = "no" ]; then OPTION="-n";
+    if [[ "$UWORDS" = "no" ]]; then OPTION="-n";
     else OPTION="-g";
     fi;
-    locale_utf8
     ;;
 
   txtu)
@@ -588,14 +601,14 @@ case "$FORMAT" in
     FORMAT="rtf";
     OPTION="-n";
     MILOCALE=$(locale -a | grep -E -i -v -m1 'utf|^C|^POSIX$')
-    if [ "$MILOCALE" = "" ]; then
+    if [[ -z "$MILOCALE" ]]; then
       echo "Error: Install a ISO-8859-1 compatible locale in your system";
       exit 1;
     fi
     export LC_CTYPE=$MILOCALE
     ;;
 
-  odtu)
+  odtu|odpu)
     OPTION="-n"
     translate_odt
     exit 0
@@ -621,7 +634,6 @@ case "$FORMAT" in
 
   wxmlu)
     OPTION="-n";
-    locale_utf8
     ;;
 
 
@@ -633,37 +645,31 @@ case "$FORMAT" in
     ;;
 esac
 
-if [ -z "$REF" ]
-then
+if [[ -z "$REF" ]]; then
     REF=$FORMAT
 fi
 
 set -e -o pipefail
 
-if [ "$FORMAT" = "none" ]; then
-    cat "$INFILE"
+if [[ "$FORMAT" = "none" ]]; then
+  cat "$INFILE"
 else
-  "$APERTIUM_PATH/apertium-des$FORMAT" "${FORMAT_OPTIONS[@]}" "$INFILE"
-fi | if [ "$TRANSLATION_MEMORY_FILE" = "" ];
-     then
-         cat
-     else
-       "$APERTIUM_PATH/lt-tmxproc" "$TMCOMPFILE"
-     fi | if [ ! -x "$DATADIR/modes/$PAIR.mode" ]; then
-              sh "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-          else
-            "$DATADIR/modes/$PAIR.mode" "$OPTION" "$OPTION_TAGGER"
-          fi | if [ "$FORMAT" = "none" ]; then
-                   if [ "$REDIR" = "" ]; then
-                       cat
-                   else
-                     cat > "$SALIDA"
-                   fi
-               else
-                 if [ "$REDIR" = "" ]; then
-                     "$APERTIUM_PATH/apertium-re$FORMAT"
-                 else
-                   "$APERTIUM_PATH/apertium-re$FORMAT" > "$SALIDA"
-                 fi
-               fi
-
+  apertium-des$FORMAT "${FORMAT_OPTIONS[@]}" "$INFILE"
+fi | if [[ -z "$TRANSLATION_MEMORY_FILE" ]];
+  then
+    cat
+  else
+    lt-tmxproc "$TMCOMPFILE"
+  fi | bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ "$FORMAT" = "none" ]]; then
+    if [[ -z "$REDIR" ]]; then
+      cat
+    else
+      cat > "$OUT_FILE"
+    fi
+  else
+    if [[ -z "$REDIR" ]]; then
+      apertium-re$FORMAT
+    else
+      apertium-re$FORMAT > "$OUT_FILE"
+    fi
+  fi
