@@ -23,6 +23,19 @@
 using namespace Apertium;
 using namespace std;
 
+std::string& pcre_version_endian() {
+  static std::string pve;
+  if (pve.empty()) {
+    pve = pcre_version();
+#ifdef WORDS_BIGENDIAN
+    pve += "-be";
+#else
+    pve += "-le";
+#endif
+  }
+  return pve;
+}
+
 ApertiumRE::ApertiumRE() :
 re(0)
 {
@@ -125,12 +138,13 @@ ApertiumRE::match(string const &str) const
   return str.substr(result[0], result[1]-result[0]);
 }
 
-void
+// Return true if something was replaced and false otherwise
+bool
 ApertiumRE::replace(string &str, string const &value) const
 {
   if(empty)
   {
-    return;
+    return false;
   }
 
   int result[3];
@@ -142,7 +156,7 @@ ApertiumRE::replace(string &str, string const &value) const
     switch(rc)
     {
       case PCRE_ERROR_NOMATCH:
-	return;
+	return false;
 
       default:
 	wcerr << L"Error: Unknown error matching regexp (code " << rc << L")" << endl;
@@ -154,4 +168,5 @@ ApertiumRE::replace(string &str, string const &value) const
   res.append(value);
   res.append(str.substr(result[1]));
   str = res;
+  return true;
 }
