@@ -18,8 +18,6 @@
 
 #include "exception_type.h"
 
-#include <sstream>
-
 namespace Apertium {
 namespace Exception {
 
@@ -29,6 +27,9 @@ namespace Exception {
     EXCEPTION_TYPE(const char *const what_) : ExceptionType(what_) {}          \
     EXCEPTION_TYPE(const std::string &what_) : ExceptionType(what_) {}         \
     EXCEPTION_TYPE(const std::stringstream &what_) : ExceptionType(what_) {}   \
+    EXCEPTION_TYPE(const wchar_t *const what_) : ExceptionType(what_) {}       \
+    EXCEPTION_TYPE(const std::wstring &what_) : ExceptionType(what_) {}        \
+    EXCEPTION_TYPE(const std::wstringstream &what_) : ExceptionType(what_) {}  \
     ~EXCEPTION_TYPE() throw() {}                                               \
   };
 
@@ -50,6 +51,7 @@ EXCEPTION(deserialise)
 EXCEPTION(optarg_eq_NULL)
 EXCEPTION(str_end_not_eq_NULL)
 EXCEPTION(ERANGE_)
+EXCEPTION(err_Exception)
 EXCEPTION(InvalidArgument)
 EXCEPTION(InvalidArgumentCombination)
 EXCEPTION(InvalidOption)
@@ -92,6 +94,20 @@ namespace wchar_t_ExceptionType {
 EXCEPTION(EILSEQ_)
 }
 
+namespace PerceptronTagger {
+EXCEPTION(CorrectAnalysisUnavailable)
+}
+
+namespace Stream {
+EXCEPTION(TheCharacterStream_not_good)
+EXCEPTION(UnexpectedAnalysis)
+EXCEPTION(UnexpectedCase)
+EXCEPTION(UnexpectedCharacter)
+EXCEPTION(UnexpectedEndOfFile)
+EXCEPTION(UnexpectedLemma)
+EXCEPTION(UnexpectedPreviousCase)
+}
+
 #undef EXCEPTION
 }
 }
