@@ -17,12 +17,10 @@
 /**
  *  Word class and MorphoStream class definitions
  *
- *  @author	Felipe Snchez-Martnez
+ *  @author	Felipe Sánchez-Martínez
  */
 
 #include <apertium/file_morpho_stream.h>
-#include <apertium/constant_manager.h>
-#include <vector>
 #include <apertium/string_utils.h>
 #include "apertium_config.h"
 #include <apertium/unlocked_cstdio.h>
