@@ -46,9 +46,8 @@ Interchunk::destroy()
 
 Interchunk::Interchunk() :
 word(0),
-blank(0),
 lword(0),
-lblank(0),
+last_lword(0),
 output(0),
 any_char(0),
 any_tag(0),
@@ -62,7 +61,7 @@ nwords(0)
   null_flush = false;
   internal_null_flush = false;
   trace = false;
-  emptyblank = "";
+  in_out = false;
 }
 
 Interchunk::~Interchunk()
@@ -92,7 +91,7 @@ Interchunk::readData(FILE *in)
   me = new MatchExe(t, finals);
 
   // attr_items
-  bool recompile_attrs = Compression::string_read(in) != string(pcre_version());
+  bool recompile_attrs = Compression::string_read(in) != pcre_version_endian();
   for(int i = 0, limit = Compression::multibyte_read(in); i != limit; i++)
   {
     string const cad_k = UtfConverter::toUtf8(Compression::wstring_read(in));
@@ -270,11 +269,19 @@ Interchunk::evalString(xmlNode *element)
         return ti.getContent();
 
       case ti_b:
-        if(ti.getPos() >= 0 && checkIndex(element, ti.getPos(), lblank))
+        if(!blank_queue.empty())
         {
-          return !blank?"":*(blank[ti.getPos()]);
+          string retblank = blank_queue.front();
+          
+          if(in_out)
+          {
+            blank_queue.pop();
+          }
+          
+          return retblank;
         }
-        else {
+        else
+        {
           return " ";
         }
         break;
@@ -406,6 +413,8 @@ Interchunk::evalString(xmlNode *element)
 void
 Interchunk::processOut(xmlNode *localroot)
 {
+  in_out = true;
+  
   for(xmlNode *i = localroot->children; i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
@@ -420,6 +429,8 @@ Interchunk::processOut(xmlNode *localroot)
       }
     }
   }
+  
+  in_out = false;
 }
 
 string
@@ -501,7 +512,13 @@ Interchunk::processLet(xmlNode *localroot)
         return;
 
       case ti_clip_tl:
-        word[ti.getPos()]->setChunkPart(attr_items[ti.getContent()], evalString(rightSide));
+      {
+        bool match = word[ti.getPos()]->setChunkPart(attr_items[ti.getContent()], evalString(rightSide));
+        if(!match && trace)
+        {
+          wcerr << "apertium-interchunk warning: <let> on line " << localroot->line << " sometimes discards its value." << endl;
+        }
+      }
         return;
 
       default:
@@ -532,8 +549,12 @@ Interchunk::processLet(xmlNode *localroot)
     }
 
 
-    word[pos]->setChunkPart(attr_items[(const char *) part],
-			    evalString(rightSide));
+    bool match = word[pos]->setChunkPart(attr_items[(const char *) part],
+					 evalString(rightSide));
+    if(!match && trace)
+    {
+      wcerr << "apertium-interchunk warning: <let> on line " << localroot->line << " sometimes discards its value." << endl;
+    }
     evalStringCache[leftSide] = TransferInstr(ti_clip_tl,
 					      (const char *) part,
 					      pos, NULL);
@@ -602,7 +623,11 @@ Interchunk::processModifyCase(xmlNode *localroot)
 
     string const result = copycase(evalString(rightSide),
 				   word[pos]->chunkPart(attr_items[(const char *) part]));
-    word[pos]->setChunkPart(attr_items[(const char *) part], result);
+    bool match = word[pos]->setChunkPart(attr_items[(const char *) part], result);
+    if(!match && trace)
+    {
+      wcerr << "apertium-interchunk warning: <modify-case> on line " << localroot->line << " sometimes discards its value." << endl;
+    }
   }
   else if(!xmlStrcmp(leftSide->name, (const xmlChar *) "var"))
   {
@@ -635,32 +660,19 @@ Interchunk::processCallMacro(xmlNode *localroot)
   {
     myword = new InterchunkWord *[npar];
   }
-  string **myblank = NULL;
-  if(npar > 0)
-  {
-    myblank = new string *[npar];
-    myblank[npar-1] = &emptyblank;
-  }
 
   int idx = 0;
-  int lastpos = 0;
   for(xmlNode *i = localroot->children; npar && i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
     {
       int pos = atoi((const char *) i->properties->children->content)-1;
       myword[idx] = word[pos];
-      if(idx-1 >= 0)
-      {
-        myblank[idx-1] = blank[lastpos];
-      }
       idx++;
-      lastpos = pos;
     }
   }
 
   swap(myword, word);
-  swap(myblank, blank);
   swap(npar, lword);
 
   for(xmlNode *i = macro->children; i != NULL; i = i->next)
@@ -672,11 +684,9 @@ Interchunk::processCallMacro(xmlNode *localroot)
   }
 
   swap(myword, word);
-  swap(myblank, blank);
   swap(npar, lword);
 
   delete[] myword;
-  delete[] myblank;
 }
 
 void
@@ -1282,6 +1292,15 @@ Interchunk::processRule(xmlNode *localroot)
       processInstruction(i);
     }
   }
+  
+  while(!blank_queue.empty()) //flush remaining blanks that are not spaces
+  {
+    if(blank_queue.front().compare(" ") != 0)
+    {
+      fputws_unlocked(UtfConverter::fromUtf8(blank_queue.front()).c_str(), output);
+    }
+    blank_queue.pop();
+  }
 }
 
 TransferToken &
@@ -1461,6 +1480,8 @@ Interchunk::interchunk(FILE *in, FILE *out)
       size_t lastrule_line = rule_lines[val-1];
       lastrule = rule_map[val-1];
       last = input_buffer.getPos();
+      
+      last_lword = tmpword.size();
 
       if(trace)
       {
@@ -1523,20 +1544,14 @@ Interchunk::applyRule()
     {
       word = new InterchunkWord *[limit];
       lword = limit;
-      if(limit != 1)
-      {
-        blank = new string *[limit - 1];
-        lblank = limit - 1;
-      }
-      else
-      {
-        blank = NULL;
-        lblank = 0;
-      }
     }
     else
     {
-      blank[i-1] = new string(UtfConverter::toUtf8(*tmpblank[i-1]));
+      if(int(blank_queue.size()) < last_lword - 1)
+      {
+        string blank_to_add = string(UtfConverter::toUtf8(*tmpblank[i-1]));
+        blank_queue.push(blank_to_add);
+      }
     }
 
     word[i] = new InterchunkWord(UtfConverter::toUtf8(*tmpword[i]));
@@ -1553,16 +1568,8 @@ Interchunk::applyRule()
     }
     delete[] word;
   }
-  if(blank)
-  {
-    for(unsigned int i = 0; i != limit - 1; i++)
-    {
-      delete blank[i];
-    }
-    delete[] blank;
-  }
+
   word = NULL;
-  blank = NULL;
   tmpword.clear();
   tmpblank.clear();
   ms.init(me->getInitial());
