@@ -34,6 +34,7 @@
 #include <map>
 #include <set>
 #include <vector>
+#include <queue>
 
 using namespace std;
 
@@ -55,8 +56,9 @@ private:
   xmlDoc *doc;
   xmlNode *root_element;
   InterchunkWord **word;
-  string **blank;
-  int lword, lblank;
+  queue <string> blank_queue;
+  int lword;
+  int last_lword;
   Buffer<TransferToken> input_buffer;
   vector<wstring *> tmpword;
   vector<wstring *> tmpblank;
@@ -73,7 +75,7 @@ private:
   bool null_flush;
   bool internal_null_flush;
   bool trace;
-  string emptyblank;
+  bool in_out;
 
   void destroy();
   void readData(FILE *input);
