@@ -65,6 +65,7 @@ InterchunkWord::operator =(InterchunkWord const &o)
 void
 InterchunkWord::init(string const &chunk)
 {
+  size_t b_end = 0;
   for(size_t i = 0; i < chunk.size(); i++)
   {
     if(chunk[i] == '\\')
@@ -77,8 +78,24 @@ InterchunkWord::init(string const &chunk)
       this->queue = chunk.substr(i);
       return;
     }
+    else if(chunk[i] == ']')
+    {
+      if(chunk[i-1] == ']')
+      {
+        b_end = i+1;
+      }
+    }
+  }
+  
+  if(b_end > 0)
+  {
+    this->wblank = chunk.substr(0, b_end);
+    this->chunk = chunk.substr(b_end);
+  }
+  else
+  {
+    this->chunk = chunk;
   }
-  this->chunk = chunk;
   this->queue = "";
 }
 
@@ -108,8 +125,14 @@ InterchunkWord::chunkPart(ApertiumRE const &part)
   }
 }
 
-void
+string
+InterchunkWord::getWblank()
+{
+  return wblank;
+}
+
+bool
 InterchunkWord::setChunkPart(ApertiumRE const &part, string const &value)
 {
-  part.replace(chunk, value);
+  return part.replace(chunk, value);
 }
