@@ -14,10 +14,10 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
-/** 
+/**
  *  Word class and MorphoStream class definitions
  *
- *  @author	Felipe Snchez-Martnez 
+ *  @author	Felipe Sánchez-Martínez
  */
 
 #ifndef __MORPHOSTREAM_H
@@ -39,76 +39,41 @@
 
 using namespace std;
 
-/** Class MorphoStream.  
- *  This class processes the output of class  yyFlexLexer (lex.yy.cc), and 
- *  builds the TaggerWord objects managed by the tagger 
+/** Class MorphoStream.
+ *  This class processes the output of class  yyFlexLexer (lex.yy.cc), and
+ *  builds the TaggerWord objects managed by the tagger
  */
 class MorphoStream {
-private:
-  bool foundEOF;
-  wstring last_string_tag;
-  bool debug;
-  FILE *input;
-  int ca_any_char;
-  int ca_any_tag;
-  int ca_kignorar;
-  int ca_kbarra;
-  int ca_kdollar;
-  int ca_kbegin;
-  int ca_kmot;
-  int ca_kmas;
-  int ca_kunknown;
-  int ca_tag_keof;
-  int ca_tag_kundef;
-
-  vector<TaggerWord *> vwords; //Vector used to implement a buffer
-                             //to treat ambiguous multiword units
-
-  MatchExe *me;
-  TaggerData *td;
-  Alphabet alphabet;
-  MatchState ms;
-
-  bool null_flush;
-  bool end_of_file;
-
-  void readRestOfWord(int &ivwords);
-  void lrlmClassify(wstring const &str, int &ivwords);
 public:
-
-   /** Constructor 
-    *  @param is the input stream.
-    */
-   MorphoStream(FILE *ftxt, bool d, TaggerData *t);
-  
-   /** 
-    *  Destructor 
-    */
-   ~MorphoStream();
-  
+   virtual ~MorphoStream();
    /** Get next word in the input stream
-    *  @return  A pointer to the next word in the input stream 
+    *  @return  A pointer to the next word in the input stream
     */
-   TaggerWord* get_next_word();  
-   
-   /** 
+   virtual TaggerWord* get_next_word() = 0;
+
+   /**
     * Set up the flag to detect '\0' characters
     * @param nf the null_flush value
     */
-   void setNullFlush(bool nf);
-   
+   virtual void setNullFlush(bool nf) = 0;
+
    /**
-    * Return true if the last reading is end of file of '\0' when null_flush 
+    * Return true if the last reading is end of file of '\0' when null_flush
     * is true
     * @returns the value of end_of_file
     */
-   bool getEndOfFile(void);
-   
+   virtual bool getEndOfFile(void) = 0;
+
    /**
     * Sets a new value for the end_of_file_flag
     * @param eof the new value for end_of_file
     */
-   void setEndOfFile(bool eof);
+   virtual void setEndOfFile(bool eof) = 0;
+
+   /**
+    * Rewind the underlying FILE* and unset the end_of_file flag
+    */
+   virtual void rewind() = 0;
 };
 
 #endif
