@@ -48,9 +48,7 @@ Postchunk::destroy()
 
 Postchunk::Postchunk() :
 word(0),
-blank(0),
 lword(0),
-lblank(0),
 output(0),
 any_char(0),
 any_tag(0),
@@ -64,6 +62,10 @@ nwords(0)
   null_flush = false;
   internal_null_flush = false;
   trace = false;
+  in_lu = false;
+  in_out = false;
+  in_let_var = false;
+  in_wblank = false;
 }
 
 Postchunk::~Postchunk()
@@ -93,7 +95,7 @@ Postchunk::readData(FILE *in)
   me = new MatchExe(t, finals);
 
   // attr_items
-  bool recompile_attrs = Compression::string_read(in) != string(pcre_version());
+  bool recompile_attrs = Compression::string_read(in) != pcre_version_endian();
   for(int i = 0, limit = Compression::multibyte_read(in); i != limit; i++)
   {
     string const cad_k = UtfConverter::toUtf8(Compression::wstring_read(in));
@@ -232,6 +234,75 @@ Postchunk::checkIndex(xmlNode *element, int index, int limit)
   return true;
 }
 
+bool
+Postchunk::gettingLemmaFromWord(string attr)
+{
+    return (attr.compare("lem") == 0 || attr.compare("lemh") == 0 || attr.compare("whole") == 0);
+}
+
+string
+Postchunk::combineWblanks(string wblank_current, string wblank_to_add)
+{
+  if(wblank_current.empty() && wblank_to_add.empty())
+  {
+    return wblank_current;
+  }
+  else if(wblank_current.empty())
+  {
+    return wblank_to_add;
+  }
+  else if(wblank_to_add.empty())
+  {
+    return wblank_current;
+  }
+  
+  string new_out_wblank;
+  for(string::const_iterator it = wblank_current.begin(); it != wblank_current.end(); it++)
+  {
+    if(*it == '\\')
+    {
+      new_out_wblank += *it;
+      it++;
+      new_out_wblank += *it;
+    }
+    else if(*it == ']')
+    {
+      if(*(it+1) == ']')
+      {
+        new_out_wblank += ';';
+        break;
+      }
+    }
+    else
+    {
+      new_out_wblank += *it;
+    }
+  }
+  
+  for(string::const_iterator it = wblank_to_add.begin(); it != wblank_to_add.end(); it++)
+  {
+    if(*it == '\\')
+    {
+      new_out_wblank += *it;
+      it++;
+      new_out_wblank += *it;
+    }
+    else if(*it == '[')
+    {
+      if(*(it+1) == '[')
+      {
+        new_out_wblank += ' ';
+        it++;
+      }
+    }
+    else
+    {
+      new_out_wblank += *it;
+    }
+  }
+  
+  return new_out_wblank;
+}
 
 string
 Postchunk::evalString(xmlNode *element)
@@ -246,6 +317,18 @@ Postchunk::evalString(xmlNode *element)
       case ti_clip_tl:
         if(checkIndex(element, ti.getPos(), lword))
         {
+          if(gettingLemmaFromWord(ti.getContent()) && lword > 1)
+          {
+            if(in_lu)
+            {
+              out_wblank = combineWblanks(out_wblank, word[ti.getPos()]->getWblank());
+            }
+            else if(in_let_var)
+            {
+              var_out_wblank[var_val] = combineWblanks(var_out_wblank[var_val], word[ti.getPos()]->getWblank());
+            }
+          }
+          
           return word[ti.getPos()]->chunkPart(attr_items[ti.getContent()]);
         }
         break;
@@ -254,6 +337,11 @@ Postchunk::evalString(xmlNode *element)
         return StringUtils::itoa_string(tmpword.size());
 
       case ti_var:
+        if(lword > 1)
+        {
+        out_wblank = combineWblanks(out_wblank, var_out_wblank[ti.getContent()]);
+        }
+        
         return variables[ti.getContent()];
 
       case ti_lit_tag:
@@ -261,11 +349,18 @@ Postchunk::evalString(xmlNode *element)
         return ti.getContent();
 
       case ti_b:
-        if(ti.getPos() >= 0 && checkIndex(element, ti.getPos(), lblank))
+        if(!blank_queue.empty())
         {
-          return !blank?"":*(blank[ti.getPos()]);
+          string retblank = blank_queue.front();
+          if(in_out)
+          {
+            blank_queue.pop();
+          }
+          
+          return retblank;
         }
-        else {
+        else
+        {
           return " ";
         }
         break;
@@ -387,6 +482,9 @@ Postchunk::evalString(xmlNode *element)
   }
   else if(!xmlStrcmp(element->name, (const xmlChar *) "lu"))
   {
+    in_lu = true;
+    out_wblank.clear();
+    
     string myword;
     for(xmlNode *i = element->children; i != NULL; i = i->next)
     {
@@ -395,10 +493,17 @@ Postchunk::evalString(xmlNode *element)
          myword.append(evalString(i));
        }
     }
+    
+    in_lu = false;
+    
+    if(lword == 1)
+    {
+      out_wblank = word[1]->getWblank();
+    }
 
     if(myword != "")
     {
-      return "^"+myword+"$";
+      return out_wblank+"^"+myword+"$";
     }
     else
     {
@@ -410,20 +515,25 @@ Postchunk::evalString(xmlNode *element)
     string value;
 
     bool first_time = true;
+    out_wblank.clear();
 
     for(xmlNode *i = element->children; i != NULL; i = i->next)
     {
       if(i->type == XML_ELEMENT_NODE)
       {
+        in_lu = true;
+        
         string myword;
 
         for(xmlNode *j = i->children; j != NULL; j = j->next)
         {
           if(j->type == XML_ELEMENT_NODE)
-	  {
+          {
             myword.append(evalString(j));
-	  }
+          }
         }
+        
+        in_lu = false;
 
 	if(!first_time)
 	{
@@ -443,10 +553,15 @@ Postchunk::evalString(xmlNode *element)
 	value.append(myword);
       }
     }
+    
+    if(lword == 1)
+    {
+      out_wblank = word[1]->getWblank();
+    }
 
     if(value != "")
     {
-      return "^"+value+"$";
+      return out_wblank+"^"+value+"$";
     }
     else
     {
@@ -466,12 +581,17 @@ Postchunk::evalString(xmlNode *element)
 void
 Postchunk::processOut(xmlNode *localroot)
 {
+  in_out = true;
+  
   for(xmlNode *i = localroot->children; i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
     {
       if(!xmlStrcmp(i->name, (const xmlChar *) "lu"))
       {
+        in_lu = true;
+        out_wblank.clear();
+        
         string myword;
         for(xmlNode *j = i->children; j != NULL; j = j->next)
         {
@@ -480,8 +600,17 @@ Postchunk::processOut(xmlNode *localroot)
             myword.append(evalString(j));
           }
         }
+        
+        in_lu = false;
+        
+        if(lword == 1)
+        {
+          out_wblank = word[1]->getWblank();
+        }
+        
         if(myword != "")
         {
+          fputws_unlocked(UtfConverter::fromUtf8(out_wblank).c_str(), output);
           fputwc_unlocked(L'^', output);
           fputws_unlocked(UtfConverter::fromUtf8(myword).c_str(), output);
           fputwc_unlocked(L'$', output);
@@ -489,38 +618,54 @@ Postchunk::processOut(xmlNode *localroot)
       }
       else if(!xmlStrcmp(i->name, (const xmlChar *) "mlu"))
       {
-        fputwc_unlocked(L'^', output);
+        string myword;
         bool first_time = true;
+        out_wblank.clear();
+        
         for(xmlNode *j = i->children; j != NULL; j = j->next)
         {
           if(j->type == XML_ELEMENT_NODE)
           {
-            string myword;
+            in_lu = true;
+            
+            string mylocalword;
             for(xmlNode *k = j->children; k != NULL; k = k->next)
             {
               if(k->type == XML_ELEMENT_NODE)
               {
-                myword.append(evalString(k));
+                mylocalword.append(evalString(k));
               }
             }
+            
+            in_lu = false;
 
             if(!first_time)
             {
-              if(myword != "")
+              if(mylocalword != "")
               {
-                fputwc_unlocked('+', output);
+                myword += '+';
               }
             }
-	    else
-	    {
-	      if(myword != "")
-	      {
-	        first_time = false;
+            else
+            {
+              if(mylocalword != "")
+              {
+                first_time = false;
               }
-	    }
-	    fputws_unlocked(UtfConverter::fromUtf8(myword).c_str(), output);
-	  }
+            }
+            
+            myword.append(mylocalword);
+          }
         }
+        
+        if(lword == 1)
+        {
+          out_wblank = word[1]->getWblank();
+        }
+
+        fputws_unlocked(UtfConverter::fromUtf8(out_wblank).c_str(), output);
+        fputwc_unlocked('^', output);
+        fputws_unlocked(UtfConverter::fromUtf8(myword).c_str(), output);
         fputwc_unlocked(L'$', output);
       }
       else // 'b'
@@ -529,6 +674,8 @@ Postchunk::processOut(xmlNode *localroot)
       }
     }
   }
+  
+  in_out = false;
 }
 
 void
@@ -609,11 +756,23 @@ Postchunk::processLet(xmlNode *localroot)
     switch(ti.getType())
     {
       case ti_var:
+        in_let_var = true;
+        var_val = ti.getContent();
+        var_out_wblank[var_val].clear();
+        
         variables[ti.getContent()] = evalString(rightSide);
+        
+        in_let_var = false;
         return;
 
       case ti_clip_tl:
-        word[ti.getPos()]->setChunkPart(attr_items[ti.getContent()], evalString(rightSide));
+      {
+        bool match = word[ti.getPos()]->setChunkPart(attr_items[ti.getContent()], evalString(rightSide));
+        if(!match && trace)
+        {
+          wcerr << "apertium-postchunk warning: <let> on line " << localroot->line << " sometimes discards its value." << endl;
+        }
+      }
         return;
 
       default:
@@ -622,8 +781,16 @@ Postchunk::processLet(xmlNode *localroot)
   }
   if(!xmlStrcmp(leftSide->name, (const xmlChar *) "var"))
   {
+    in_let_var = true;
+    
     string const val = (const char *) leftSide->properties->children->content;
+    
+    var_val = val;
+    var_out_wblank[var_val].clear();
+    
     variables[val] = evalString(rightSide);
+    
+    in_let_var = false;
     evalStringCache[leftSide] = TransferInstr(ti_var, val, 0);
   }
   else if(!xmlStrcmp(leftSide->name, (const xmlChar *) "clip"))
@@ -644,8 +811,12 @@ Postchunk::processLet(xmlNode *localroot)
     }
 
 
-    word[pos]->setChunkPart(attr_items[(const char *) part],
-			    evalString(rightSide));
+    bool match = word[pos]->setChunkPart(attr_items[(const char *) part],
+					 evalString(rightSide));
+    if(!match && trace)
+    {
+      wcerr << "apertium-postchunk warning: <let> on line " << localroot->line << " sometimes discards its value." << endl;
+    }
     evalStringCache[leftSide] = TransferInstr(ti_clip_tl, (const char *) part,
 					      pos, NULL);
   }
@@ -668,7 +839,10 @@ Postchunk::processAppend(xmlNode *localroot)
   {
     if(i->type == XML_ELEMENT_NODE)
     {
+      in_let_var = true;
+      var_val = name;
       variables[name].append(evalString(i));
+      in_let_var = false;
     }
   }
 }
@@ -713,8 +887,12 @@ Postchunk::processModifyCase(xmlNode *localroot)
 
     string const result = copycase(evalString(rightSide),
 				   word[pos]->chunkPart(attr_items[(const char *) part]));
-    word[pos]->setChunkPart(attr_items[(const char *) part], result);
+    bool match = word[pos]->setChunkPart(attr_items[(const char *) part], result);
 
+    if(!match && trace)
+    {
+      wcerr << "apertium-postchunk warning: <modify-case> on line " << localroot->line << " sometimes discards its value." << endl;
+    }
   }
   else if(!xmlStrcmp(leftSide->name, (const xmlChar *) "var"))
   {
@@ -750,17 +928,11 @@ Postchunk::processCallMacro(xmlNode *localroot)
   {
     myword = new InterchunkWord *[npar+1];
   }
-  string **myblank = NULL;
-  if(npar > 0)
-  {
-    myblank = new string *[npar];
-  }
 
   myword[0] = word[0];
 
   bool indexesOK = true;
   int idx = 1;
-  int lastpos = 0;
   for(xmlNode *i = localroot->children; i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
@@ -771,18 +943,11 @@ Postchunk::processCallMacro(xmlNode *localroot)
         pos = 1;
       }
       myword[idx] = word[pos];
-      if(blank)
-      {
-        myblank[idx-1] = blank[lastpos];
-      }
-
       idx++;
-      lastpos = pos;
     }
   }
 
   swap(myword, word);
-  swap(myblank, blank);
   swap(npar, lword);
 
   if(indexesOK) {
@@ -799,11 +964,9 @@ Postchunk::processCallMacro(xmlNode *localroot)
   }
 
   swap(myword, word);
-  swap(myblank, blank);
   swap(npar, lword);
 
   delete[] myword;
-  delete[] myblank;
 }
 
 void
@@ -1445,6 +1608,15 @@ Postchunk::processRule(xmlNode *localroot)
       processInstruction(i);
     }
   }
+  
+  while(!blank_queue.empty()) //flush remaining blanks that are not spaces
+  {
+    if(blank_queue.front().compare(" ") != 0)
+    {
+      fputws_unlocked(UtfConverter::fromUtf8(blank_queue.front()).c_str(), output);
+    }
+    blank_queue.pop();
+  }
 }
 
 TransferToken &
@@ -1685,22 +1857,10 @@ Postchunk::applyRule()
 
   for(unsigned int i = 1, limit = tmpword.size()+1; i != limit; i++)
   {
-    if(i == 1)
+    if(i != 1)
     {
-      if(limit != 2)
-      {
-        blank = new string *[limit - 2];
-        lblank = limit - 3;
-      }
-      else
-      {
-        blank = NULL;
-        lblank = 0;
-      }
-    }
-    else
-    {
-      blank[i-2] = new string(UtfConverter::toUtf8(*tmpblank[i-1]));
+      string blank_to_add = string(UtfConverter::toUtf8(*tmpblank[i-1]));
+      blank_queue.push(blank_to_add);
     }
 
     word[i] = new InterchunkWord(UtfConverter::toUtf8(*tmpword[i-1]));
@@ -1717,16 +1877,7 @@ Postchunk::applyRule()
     }
     delete[] word;
   }
-  if(blank)
-  {
-    for(unsigned int i = 0, limit = tmpword.size() - 1; i != limit; i++)
-    {
-      delete blank[i];
-    }
-    delete[] blank;
-  }
   word = NULL;
-  blank = NULL;
 
   for(unsigned int i = 0, limit = tmpword.size(); i != limit; i++)
   {
@@ -2067,27 +2218,115 @@ Postchunk::splitWordsAndBlanks(wstring const &chunk, vector<wstring *> &words,
     }
     else if(chunk[i] == L'[')
     {
-      if (!(lastblank && blanks.back()))
+      if(chunk[i+1] == L'[') //wordbound blank
       {
-        blanks.push_back(new wstring());
-      }
-      wstring &ref = *(blanks.back());
-      ref += L'[';
-      while(chunk[++i] != L']')
-      {
-        if(chunk[i] == L'\\')
+        if(!lastblank)
         {
-          ref += L'\\';
-          ref += chunk[++i];
+          blanks.push_back(new wstring(L""));
         }
-        else
-        {
-          ref += chunk[i];
-        }
-      }
-      ref += chunk[i];
+        lastblank = false;
+        wstring *myword = new wstring();
+        wstring &ref = *myword;
 
-      lastblank = true;
+        while(true)
+        {
+          if(chunk[i] == L'\\')
+          {
+            ref += L'\\';
+            ref += chunk[++i];
+          }
+          else if(chunk[i] == L']' && chunk[i-1] == L']')
+          {
+            ref += chunk[i];
+            i++; //i->"^"
+            break;
+          }
+          else
+          {
+            ref += chunk[i];
+          }
+          
+          i++;
+        }
+        
+        while(chunk[++i] != L'$')
+        {
+          if(chunk[i] == L'\\')
+          {
+            ref += L'\\';
+            ref += chunk[++i];
+          }
+          else if(chunk[i] == L'<')
+          {
+            if(iswdigit(chunk[i+1]))
+            {
+              // replace tag
+              unsigned long value = wcstoul(chunk.c_str()+i+1,
+                                            NULL, 0) - 1;
+              if(vectags.size() > value)
+              {
+                ref.append(vectags[value]);
+              }
+              while(chunk[++i] != L'>');
+            }
+            else
+            {
+              ref += L'<';
+              while(chunk[++i] != L'>') ref += chunk[i];
+              ref += L'>';
+            }
+          }
+          else
+          {
+            if(uppercase_all)
+            {
+              ref += towupper(chunk[i]);
+            }
+            else if(uppercase_first)
+            {
+              if(iswalnum(chunk[i]))
+              {
+                ref += towupper(chunk[i]);
+                uppercase_first = false;
+              }
+              else
+              {
+                ref += chunk[i];
+              }
+            }
+            else
+            {
+              ref += chunk[i];
+            }
+          }
+        }
+
+        words.push_back(myword);
+      }
+      else
+      {
+        if (!(lastblank && blanks.back()))
+        {
+          blanks.push_back(new wstring());
+        }
+        wstring &ref = *(blanks.back());
+        ref += L'[';
+        while(chunk[++i] != L']')
+        {
+          if(chunk[i] == L'\\')
+          {
+            ref += L'\\';
+            ref += chunk[++i];
+          }
+          else
+          {
+            ref += chunk[i];
+          }
+        }
+        ref += chunk[i];
+
+        lastblank = true;
+      }
     }
     else
     {
