@@ -13,6 +13,7 @@
 
 #include <apertium/tmx_words.h> // For SentenceList
 #include <apertium/tmx_dictionary.h> // For FrequencyMap
+#include <apertium/string_utils.h>
 
 #include <iostream>
 #include <map>
@@ -20,7 +21,7 @@
 #include <algorithm>
 
 // Copypaste-elve. TODO Elhelyezni.
-#define massert(e) if (!(e)) { std::cerr << #e << " failed" << std::endl; throw "assert"; }
+#define massert(e) if (!(e)) { std::wcerr << #e << " failed" << std::endl; throw "assert"; }
 
 std::ostream& operator<<( std::ostream& os, std::pair<int,int> p )
 {
@@ -240,7 +241,7 @@ void trelliToLadder( const TrelliMatrix& trellis, Trail& bestTrail )
 
   bool logging = false;
 
-  if (logging) std::cerr << std::endl;
+  if (logging) std::wcerr << std::endl;
 
   bool over = false;
   bool hopelesslyBadTrail = false;
@@ -303,7 +304,7 @@ void trelliToLadder( const TrelliMatrix& trellis, Trail& bestTrail )
 
     if (logging)
     {
-      std::cerr << huPos << " \t" << enPos << std::endl;
+      std::wcerr << huPos << " \t" << enPos << std::endl;
     }
 
   }
@@ -313,7 +314,7 @@ void trelliToLadder( const TrelliMatrix& trellis, Trail& bestTrail )
     bestTrail.clear();
     bestTrail.push_back(std::make_pair(huBookSize,enBookSize));
     bestTrail.push_back(std::make_pair(0,0));
-    std::cerr << "Error: hopelessly bad trail." << std::endl;
+    std::wcerr << "Error: hopelessly bad trail." << std::endl;
   }
 
   std::reverse(bestTrail.begin(),  bestTrail.end()  );
@@ -334,11 +335,11 @@ void align( const AlignMatrix& w, const SentenceValues& huLength, const Sentence
 
   buildDynProgMatrix( w, huLength, enLength, v, trellis );
 
-//  std::cerr << "Matrix built." << std::endl;
+//  std::wcerr << "Matrix built." << std::endl;
 
   trelliToLadder( trellis, bestTrail );
 
-//  std::cerr << "Trail found." << std::endl;
+//  std::wcerr << "Trail found." << std::endl;
 }
 
 
@@ -382,10 +383,10 @@ double scoreTrailOrBisentenceList( const Trail& trailAuto, const Trail& trailHan
 {
   int score = countIntersectionOfTrails( trailAuto, trailHand );
 
-  std::cerr << trailAuto.size()-score << " misaligned out of " << trailHand.size() << " correct items, "
+  std::wcerr << trailAuto.size()-score << " misaligned out of " << trailHand.size() << " correct items, "
     << trailAuto.size() << " bets." << std::endl;
 
-  std::cerr << "Precision: " << 1.0*score/trailAuto.size() 
+  std::wcerr << "Precision: " << 1.0*score/trailAuto.size()
     << ", Recall: " << 1.0*score/trailHand.size() << std::endl;
 
   double ratio = 1.0*(trailAuto.size()-score)/trailAuto.size();
@@ -493,7 +494,7 @@ bool borderDetailedAlignMatrix( AlignMatrix& alignMatrix, const Trail& trail, in
       }
     }
 
-    std::cerr << numberOfEvaluatedItems << " items inside the border." << std::endl;
+    std::wcerr << numberOfEvaluatedItems << " items inside the border." << std::endl;
   }
 
   return true;
