@@ -51,9 +51,8 @@ Transfer::destroy()
 
 Transfer::Transfer() :
 word(0),
-blank(0),
 lword(0),
-lblank(0),
+last_lword(0),
 output(0),
 any_char(0),
 any_tag(0),
@@ -71,7 +70,10 @@ nwords(0)
   internal_null_flush = false;
   trace = false;
   trace_att = false;
-  emptyblank = "";
+  in_lu = false;
+  in_let_var = false;
+  in_out = false;
+  in_wblank = false;
 }
 
 Transfer::~Transfer()
@@ -101,7 +103,7 @@ Transfer::readData(FILE *in)
   me = new MatchExe(t, finals);
 
   // attr_items
-  bool recompile_attrs = Compression::string_read(in) != string(pcre_version());
+  bool recompile_attrs = Compression::string_read(in) != pcre_version_endian();
   for(int i = 0, limit = Compression::multibyte_read(in); i != limit; i++)
   {
     string const cad_k = UtfConverter::toUtf8(Compression::wstring_read(in));
@@ -290,6 +292,75 @@ Transfer::checkIndex(xmlNode *element, int index, int limit)
   return true;
 }
 
+bool
+Transfer::gettingLemmaFromWord(string attr)
+{
+    return (attr.compare("lem") == 0 || attr.compare("lemh") == 0 || attr.compare("whole") == 0);
+}
+
+string
+Transfer::combineWblanks(string wblank_current, string wblank_to_add)
+{
+  if(wblank_current.empty() && wblank_to_add.empty())
+  {
+    return wblank_current;
+  }
+  else if(wblank_current.empty())
+  {
+    return wblank_to_add;
+  }
+  else if(wblank_to_add.empty())
+  {
+    return wblank_current;
+  }
+  
+  string new_out_wblank;
+  for(string::const_iterator it = wblank_current.begin(); it != wblank_current.end(); it++)
+  {
+    if(*it == '\\')
+    {
+      new_out_wblank += *it;
+      it++;
+      new_out_wblank += *it;
+    }
+    else if(*it == ']')
+    {
+      if(*(it+1) == ']')
+      {
+        new_out_wblank += ';';
+        break;
+      }
+    }
+    else
+    {
+      new_out_wblank += *it;
+    }
+  }
+  
+  for(string::const_iterator it = wblank_to_add.begin(); it != wblank_to_add.end(); it++)
+  {
+    if(*it == '\\')
+    {
+      new_out_wblank += *it;
+      it++;
+      new_out_wblank += *it;
+    }
+    else if(*it == '[')
+    {
+      if(*(it+1) == '[')
+      {
+        new_out_wblank += ' ';
+        it++;
+      }
+    }
+    else
+    {
+      new_out_wblank += *it;
+    }
+  }
+  
+  return new_out_wblank;
+}
 
 string
 Transfer::evalString(xmlNode *element)
@@ -304,6 +375,18 @@ Transfer::evalString(xmlNode *element)
       case ti_clip_sl:
         if(checkIndex(element, ti.getPos(), lword))
         {
+          if(gettingLemmaFromWord(ti.getContent()) && last_lword > 1)
+          {
+            if(in_lu)
+            {
+              out_wblank = combineWblanks(out_wblank, word[ti.getPos()]->getWblank());
+            }
+            else if(in_let_var)
+            {
+              var_out_wblank[var_val] = combineWblanks(var_out_wblank[var_val], word[ti.getPos()]->getWblank());
+            }
+          }
+          
           return word[ti.getPos()]->source(attr_items[ti.getContent()], ti.getCondition());
         }
         break;
@@ -311,6 +394,18 @@ Transfer::evalString(xmlNode *element)
       case ti_clip_tl:
         if(checkIndex(element, ti.getPos(), lword))
         {
+          if(gettingLemmaFromWord(ti.getContent()) && last_lword > 1)
+          {
+            if(in_lu)
+            {
+              out_wblank = combineWblanks(out_wblank, word[ti.getPos()]->getWblank());
+            }
+            else if(in_let_var)
+            {
+              var_out_wblank[var_val] = combineWblanks(var_out_wblank[var_val], word[ti.getPos()]->getWblank());
+            }
+          }
+            
           return word[ti.getPos()]->target(attr_items[ti.getContent()], ti.getCondition());
         }
         break;
@@ -365,6 +460,10 @@ Transfer::evalString(xmlNode *element)
         break;
 
       case ti_var:
+        if(last_lword > 1)
+        {
+          out_wblank = combineWblanks(out_wblank, var_out_wblank[ti.getContent()]);
+        }
         return variables[ti.getContent()];
 
       case ti_lit_tag:
@@ -372,11 +471,18 @@ Transfer::evalString(xmlNode *element)
         return ti.getContent();
 
       case ti_b:
-        if(ti.getPos() >= 0 && checkIndex(element, ti.getPos(), lblank))
+        if(!blank_queue.empty())
         {
-          return !blank?"":*(blank[ti.getPos()]);
+          string retblank = blank_queue.front();
+          if(in_out)
+          {
+            blank_queue.pop();
+          }
+          
+          return retblank;
         }
-        else {
+        else
+        {
           return " ";
         }
         break;
@@ -488,15 +594,7 @@ Transfer::evalString(xmlNode *element)
   }
   else if(!xmlStrcmp(element->name, (const xmlChar *) "b"))
   {
-    if(element->properties == NULL)
-    {
-      evalStringCache[element] = TransferInstr(ti_b, " ", -1);
-    }
-    else
-    {
-      int pos = atoi((const char *) element->properties->children->content) - 1;
-      evalStringCache[element] = TransferInstr(ti_b, "", pos);
-    }
+    evalStringCache[element] = TransferInstr(ti_b, " ", -1);
   }
   else if(!xmlStrcmp(element->name, (const xmlChar *) "get-case-from"))
   {
@@ -565,6 +663,9 @@ Transfer::evalString(xmlNode *element)
   }
   else if(!xmlStrcmp(element->name, (const xmlChar *) "lu"))
   {
+    in_lu = true;
+    out_wblank.clear();
+      
     string myword;
     for(xmlNode *i = element->children; i != NULL; i = i->next)
     {
@@ -573,10 +674,24 @@ Transfer::evalString(xmlNode *element)
          myword.append(evalString(i));
        }
     }
-
+    
+    in_lu = false;
+    
+    if(last_lword == 1)
+    {
+      out_wblank = word[0]->getWblank();
+    }
+      
     if(myword != "")
     {
-      return "^"+myword+"$";
+      if(myword[0] != L'[' || myword[1] != L'[')
+      {
+        return out_wblank+"^"+myword+"$";
+      }
+      else
+      {
+        return myword+"$";
+      }
     }
     else
     {
@@ -588,20 +703,25 @@ Transfer::evalString(xmlNode *element)
     string value;
 
     bool first_time = true;
+    out_wblank.clear();
 
     for(xmlNode *i = element->children; i != NULL; i = i->next)
     {
       if(i->type == XML_ELEMENT_NODE)
       {
+        in_lu = true;
+        
         string myword;
 
         for(xmlNode *j = i->children; j != NULL; j = j->next)
         {
           if(j->type == XML_ELEMENT_NODE)
-	  {
+          {
             myword.append(evalString(j));
-	  }
+          }
         }
+        
+        in_lu = false;
 
 	if(!first_time)
 	{
@@ -622,9 +742,14 @@ Transfer::evalString(xmlNode *element)
       }
     }
 
+    if(last_lword == 1)
+    {
+      out_wblank = word[0]->getWblank();
+    }
+    
     if(value != "")
     {
-      return "^"+value+"$";
+      return out_wblank+"^"+value+"$";
     }
     else
     {
@@ -647,6 +772,8 @@ Transfer::evalString(xmlNode *element)
 void
 Transfer::processOut(xmlNode *localroot)
 {
+  in_out = true;
+  
   for(xmlNode *i = localroot->children; i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
@@ -655,56 +782,91 @@ Transfer::processOut(xmlNode *localroot)
       {
         if(!xmlStrcmp(i->name, (const xmlChar *) "lu"))
         {
-  	  string myword;
-	  for(xmlNode *j = i->children; j != NULL; j = j->next)
-	  {
-	    if(j->type == XML_ELEMENT_NODE)
-	    {
-	      myword.append(evalString(j));
+          in_lu = true;
+          out_wblank.clear();
+            
+          string myword;
+          for(xmlNode *j = i->children; j != NULL; j = j->next)
+          {
+            if(j->type == XML_ELEMENT_NODE)
+            {
+              myword.append(evalString(j));
             }
-	  }
-	  if(myword != "")
-	  {
-  	    fputwc_unlocked(L'^', output);
-   	    fputws_unlocked(UtfConverter::fromUtf8(myword).c_str(), output);
-	    fputwc_unlocked(L'$', output);
+          }
+            
+          in_lu = false;
+          
+          if(last_lword == 1)
+          {
+            out_wblank = word[0]->getWblank();
+          }
+
+          if(myword != "")
+          {
+            if(myword[0] != L'[' || myword[1] != L'[')
+            {
+              fputws_unlocked(UtfConverter::fromUtf8(out_wblank).c_str(), output);
+              fputwc_unlocked(L'^', output);
+            }
+            
+            fputws_unlocked(UtfConverter::fromUtf8(myword).c_str(), output);
+            fputwc_unlocked(L'$', output);
           }
         }
         else if(!xmlStrcmp(i->name, (const xmlChar *) "mlu"))
         {
-	  fputwc_unlocked('^', output);
-	  bool first_time = true;
-	  for(xmlNode *j = i->children; j != NULL; j = j->next)
-	  {
-	    if(j->type == XML_ELEMENT_NODE)
-	    {
-              string myword;
-	      for(xmlNode *k = j->children; k != NULL; k = k->next)
-	      {
-	        if(k->type == XML_ELEMENT_NODE)
-	        {
-                  myword.append(evalString(k));
-	        }
-	      }
+          string myword;
+          bool first_time = true;
+          out_wblank.clear();
+          
+          for(xmlNode *j = i->children; j != NULL; j = j->next)
+          {
+            if(j->type == XML_ELEMENT_NODE)
+            {
+              in_lu = true;
+              
+              string mylocalword;
+              for(xmlNode *k = j->children; k != NULL; k = k->next)
+              {
+                if(k->type == XML_ELEMENT_NODE)
+                {
+                  mylocalword.append(evalString(k));
+                }
+              }
+              
+              in_lu = false;
 
-	      if(!first_time)
-	      {
-	        if(myword != "" && myword[0] != '#')  //'+#' problem
-	        {
-	          fputwc_unlocked(L'+', output);
+              if(!first_time)
+              {
+                if(mylocalword != "" && mylocalword[0] != '#')  //'+#' problem
+                {
+                  myword += '+';
                 }
-	      }
-	      else
-	      {
-	        if(myword != "")
-	        {
-	          first_time = false;
+              }
+              else
+              {
+                if(mylocalword != "")
+                {
+                  first_time = false;
                 }
-	      }
-	      fputws_unlocked(UtfConverter::fromUtf8(myword).c_str(), output);
-	    }
-	  }
-	  fputwc_unlocked(L'$', output);
+              }
+              
+              myword.append(mylocalword);
+            }
+          }
+          
+          if(last_lword == 1)
+          {
+            out_wblank = word[0]->getWblank();
+          }
+          
+          if(myword != "")
+          {
+            fputws_unlocked(UtfConverter::fromUtf8(out_wblank).c_str(), output);
+            fputwc_unlocked('^', output);
+            fputws_unlocked(UtfConverter::fromUtf8(myword).c_str(), output);
+            fputwc_unlocked(L'$', output);
+          }
         }
         else // 'b'
         {
@@ -725,6 +887,8 @@ Transfer::processOut(xmlNode *localroot)
       }
     }
   }
+  
+  in_out = false;
 }
 
 string
@@ -795,6 +959,9 @@ Transfer::processChunk(xmlNode *localroot)
       }
       else if(!xmlStrcmp(i->name, (const xmlChar *) "lu"))
       {
+        in_lu = true;
+        out_wblank.clear();
+          
         string myword;
         for(xmlNode *j = i->children; j != NULL; j = j->next)
         {
@@ -803,8 +970,17 @@ Transfer::processChunk(xmlNode *localroot)
             myword.append(evalString(j));
           }
         }
+        
+        in_lu = false;
+        
+        if(last_lword == 1)
+        {
+          out_wblank = word[0]->getWblank();
+        }
+          
         if(myword != "")
         {
+          result.append(out_wblank);
           result.append("^");
           result.append(myword);
           result.append("$");
@@ -814,11 +990,16 @@ Transfer::processChunk(xmlNode *localroot)
       {
         bool first_time = true;
         string myword;
+        
+        out_wblank.clear();
+        
         for(xmlNode *j = i->children; j != NULL; j = j->next)
         {
           string mylocalword;
           if(j->type == XML_ELEMENT_NODE)
           {
+            in_lu = true;
+            
             for(xmlNode *k = j->children; k != NULL; k = k->next)
             {
               if(k->type == XML_ELEMENT_NODE)
@@ -826,6 +1007,8 @@ Transfer::processChunk(xmlNode *localroot)
                 mylocalword.append(evalString(k));
               }
             }
+            
+            in_lu = false;
 
             if(!first_time)
             {
@@ -841,8 +1024,15 @@ Transfer::processChunk(xmlNode *localroot)
           }
           myword.append(mylocalword);
         }
+        
+        if(last_lword == 1)
+        {
+          out_wblank = word[0]->getWblank();
+        }
+        
         if(myword != "")
         {
+          result.append(out_wblank);
           result.append("^");
           result.append(myword);
           result.append("$");
@@ -966,24 +1156,44 @@ Transfer::processLet(xmlNode *localroot)
     switch(ti.getType())
     {
       case ti_var:
+        in_let_var = true;
+        var_val = ti.getContent();
+
+        var_out_wblank[var_val].clear();
+        
         variables[ti.getContent()] = evalString(rightSide);
+          
+        in_let_var = false;
+        
         return;
 
       case ti_clip_sl:
         if (checkIndex(leftSide, ti.getPos(), lword)) {
-          word[ti.getPos()]->setSource(attr_items[ti.getContent()], evalString(rightSide), ti.getCondition());
+          bool match = word[ti.getPos()]->setSource(attr_items[ti.getContent()], evalString(rightSide), ti.getCondition());
+          if (!match && trace)
+          {
+            wcerr << "apertium-transfer warning: <let> on line " << localroot->line << " sometimes discards its value." << endl;
+          }
         }
         return;
 
       case ti_clip_tl:
         if (checkIndex(leftSide, ti.getPos(), lword)) {
-          word[ti.getPos()]->setTarget(attr_items[ti.getContent()], evalString(rightSide), ti.getCondition());
+          bool match = word[ti.getPos()]->setTarget(attr_items[ti.getContent()], evalString(rightSide), ti.getCondition());
+          if (!match && trace)
+          {
+            wcerr << "apertium-transfer warning: <let> on line " << localroot->line << " sometimes discards its value." << endl;
+          }
         }
         return;
 
       case ti_clip_ref:
         if (checkIndex(leftSide, ti.getPos(), lword)) {
-          word[ti.getPos()]->setReference(attr_items[ti.getContent()], evalString(rightSide), ti.getCondition());
+          bool match = word[ti.getPos()]->setReference(attr_items[ti.getContent()], evalString(rightSide), ti.getCondition());
+          if (!match && trace)
+          {
+            wcerr << "apertium-transfer warning: <let> on line " << localroot->line << " sometimes discards its value." << endl;
+          }
         }
         return;
 
@@ -993,8 +1203,16 @@ Transfer::processLet(xmlNode *localroot)
   }
   if(leftSide->name != NULL && !xmlStrcmp(leftSide->name, (const xmlChar *) "var"))
   {
+    in_let_var = true;
+    
     string const val = (const char *) leftSide->properties->children->content;
+    
+    var_val = val;
+    var_out_wblank[var_val].clear();
+    
     variables[val] = evalString(rightSide);
+      
+    in_let_var = false;
     evalStringCache[leftSide] = TransferInstr(ti_var, val, 0);
   }
   else if(!xmlStrcmp(leftSide->name, (const xmlChar *) "clip"))
@@ -1041,17 +1259,29 @@ Transfer::processLet(xmlNode *localroot)
 
     if(!xmlStrcmp(side, (const xmlChar *) "tl"))
     {
-      word[pos]->setTarget(attr_items[(const char *) part], evalString(rightSide), queue);
+      bool match = word[pos]->setTarget(attr_items[(const char *) part], evalString(rightSide), queue);
+      if(!match && trace)
+      {
+        wcerr << "apertium-transfer warning: <let> on line " << localroot->line << " sometimes discards its value." << endl;
+      }
       evalStringCache[leftSide] = TransferInstr(ti_clip_tl, (const char *) part, pos, NULL, queue);
     }
     else if(!xmlStrcmp(side, (const xmlChar *) "ref"))
     {
-      word[pos]->setReference(attr_items[(const char *) part], evalString(rightSide), queue);
+      bool match = word[pos]->setReference(attr_items[(const char *) part], evalString(rightSide), queue);
+      if(!match && trace)
+      {
+        wcerr << "apertium-transfer warning: <let> on line " << localroot->line << " sometimes discards its value." << endl;
+      }
       evalStringCache[leftSide] = TransferInstr(ti_clip_ref, (const char *) part, pos, NULL, queue);
     }
     else
     {
-      word[pos]->setSource(attr_items[(const char *) part], evalString(rightSide), queue);
+      bool match = word[pos]->setSource(attr_items[(const char *) part], evalString(rightSide), queue);
+      if(!match && trace)
+      {
+        wcerr << "apertium-transfer warning: <let> on line " << localroot->line << " sometimes discards its value." << endl;
+      }
       evalStringCache[leftSide] = TransferInstr(ti_clip_sl, (const char *) part, pos, NULL, queue);
     }
   }
@@ -1074,7 +1304,10 @@ Transfer::processAppend(xmlNode *localroot)
   {
     if(i->type == XML_ELEMENT_NODE)
     {
+      in_let_var = true;
+      var_val = name;
       variables[name].append(evalString(i));
+      in_let_var = false;
     }
   }
 }
@@ -1137,19 +1370,31 @@ Transfer::processModifyCase(xmlNode *localroot)
     {
       string const result = copycase(evalString(rightSide),
 				      word[pos]->source(attr_items[(const char *) part], queue));
-      word[pos]->setSource(attr_items[(const char *) part], result);
+      bool match = word[pos]->setSource(attr_items[(const char *) part], result);
+      if(!match && trace)
+      {
+        wcerr << "apertium-transfer warning: <modify-case> on line " << localroot->line << " sometimes discards its value." << endl;
+      }
     }
     else if(!xmlStrcmp(side, (const xmlChar *) "ref"))
     {
       string const result = copycase(evalString(rightSide),
               word[pos]->reference(attr_items[(const char *) part], queue));
-      word[pos]->setReference(attr_items[(const char *) part], result);
+      bool match = word[pos]->setReference(attr_items[(const char *) part], result);
+      if(!match && trace)
+      {
+        wcerr << "apertium-transfer warning: <modify-case> on line " << localroot->line << " sometimes discards its value." << endl;
+      }
     }
     else
     {
       string const result = copycase(evalString(rightSide),
 				     word[pos]->target(attr_items[(const char *) part], queue));
-      word[pos]->setTarget(attr_items[(const char *) part], result);
+      bool match = word[pos]->setTarget(attr_items[(const char *) part], result);
+      if(!match && trace)
+      {
+        wcerr << "apertium-transfer warning: <modify-case> on line " << localroot->line << " sometimes discards its value." << endl;
+      }
     }
   }
   else if(!xmlStrcmp(leftSide->name, (const xmlChar *) "var"))
@@ -1184,15 +1429,8 @@ Transfer::processCallMacro(xmlNode *localroot)
     myword = new TransferWord *[npar];
     std::fill(myword, myword+npar, (TransferWord *)(0));
   }
-  string **myblank = NULL;
-  if(npar > 0)
-  {
-    myblank = new string *[npar];
-    myblank[npar-1] = &emptyblank;
-  }
-
+  
   int idx = 0;
-  int lastpos = 0;
   for(xmlNode *i = localroot->children; npar && i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
@@ -1203,17 +1441,12 @@ Transfer::processCallMacro(xmlNode *localroot)
       }
       int pos = atoi((const char *) i->properties->children->content)-1;
       myword[idx] = word[pos];
-      if(idx-1 >= 0)
-      {
-        myblank[idx-1] = blank[lastpos];
-      }
+
       idx++;
-      lastpos = pos;
     }
   }
 
   swap(myword, word);
-  swap(myblank, blank);
   swap(npar, lword);
 
   for(xmlNode *i = macro->children; i != NULL; i = i->next)
@@ -1225,11 +1458,9 @@ Transfer::processCallMacro(xmlNode *localroot)
   }
 
   swap(myword, word);
-  swap(myblank, blank);
   swap(npar, lword);
 
   delete[] myword;
-  delete[] myblank;
 }
 
 int
@@ -1856,6 +2087,16 @@ Transfer::processRule(xmlNode *localroot)
       }
     }
   }
+  
+  while(!blank_queue.empty()) //flush remaining blanks that are not spaces
+  {
+    if(blank_queue.front().compare(" ") != 0)
+    {
+      fputws_unlocked(UtfConverter::fromUtf8(blank_queue.front()).c_str(), output);
+    }
+    blank_queue.pop();
+  }
+  
   return words_to_consume;
 }
 
@@ -1873,8 +2114,38 @@ Transfer::readToken(FILE *in)
     int val = fgetwc_unlocked(in);
     if(feof(in) || (val == 0 && internal_null_flush))
     {
+      in_wblank = false;
       return input_buffer.add(TransferToken(content, tt_eof));
     }
+    if(in_wblank)
+    {
+      content = L"[[";
+      content+= wchar_t(val);
+      
+      while(true)
+      {
+        int val3 = fgetwc_unlocked(in);
+        if(val3 == L'\\')
+        {
+          content += L'\\';
+          content += wchar_t(fgetwc_unlocked(in));
+        }
+        else if(val3 == L'$') //[[..]]^..$ is the LU
+        {
+          in_wblank = false;
+          return input_buffer.add(TransferToken(content, tt_word));
+        }
+        else if(val3 == L'\0' && null_flush)
+        {
+          in_wblank = false;
+          fflush(output);
+        }
+        else
+        {
+          content += wchar_t(val3);
+        }
+      }
+    }
     if(val == '\\')
     {
       content += L'\\';
@@ -1885,21 +2156,28 @@ Transfer::readToken(FILE *in)
       content += L'[';
       while(true)
       {
-	int val2 = fgetwc_unlocked(in);
-	if(val2 == L'\\')
-	{
-	  content += L'\\';
-	  content += wchar_t(fgetwc_unlocked(in));
-	}
-	else if(val2 == L']')
-	{
-	  content += L']';
-	  break;
-	}
-	else
-	{
-	  content += wchar_t(val2);
-	}
+        int val2 = fgetwc_unlocked(in);
+        if(val2 == L'\\')
+        {
+          content += L'\\';
+          content += wchar_t(fgetwc_unlocked(in));
+        }
+        else if(val2 == L'[')
+        { //wordbound blank
+          in_wblank = true;
+          content.pop_back();
+          
+          return input_buffer.add(TransferToken(content, tt_blank));
+        }
+        else if(val2 == L']')
+        {
+          content += L']';
+          break;
+        }
+        else
+        {
+          content += wchar_t(val2);
+        }
       }
     }
     else if(val == L'$')
@@ -1912,6 +2190,7 @@ Transfer::readToken(FILE *in)
     }
     else if(val == L'\0' && null_flush)
     {
+      in_wblank = false;
       fflush(output);
     }
     else
@@ -1945,6 +2224,13 @@ Transfer::setTraceATT(bool trace)
   this->trace_att = trace;
 }
 
+void
+Transfer::tmp_clear()
+{
+  tmpblank.clear();
+  tmpword.clear();
+}
+
 void
 Transfer::transfer_wrapper_null_flush(FILE *in, FILE *out)
 {
@@ -1953,6 +2239,7 @@ Transfer::transfer_wrapper_null_flush(FILE *in, FILE *out)
 
   while(!feof(in))
   {
+    tmp_clear();
     transfer(in, out);
     fputwc_unlocked(L'\0', out);
     int code = fflush(out);
@@ -1978,6 +2265,7 @@ Transfer::transfer(FILE *in, FILE *out)
   unsigned int prev_last = last;
   int lastrule_id = -1;
   set<int> banned_rules;
+  in_wblank = false;
 
   output = out;
   ms.init(me->getInitial());
@@ -2072,6 +2360,7 @@ Transfer::transfer(FILE *in, FILE *out)
           }
 
           pair<wstring, int> tr;
+          wstring tr_wblank;
           if(useBilingual && preBilingual == false)
           {
 	    if(isExtended && (*tmpword[0])[0] == L'*')
@@ -2096,6 +2385,7 @@ Transfer::transfer(FILE *in, FILE *out)
             wstring sl;
             wstring tl;
             wstring ref;
+            wstring wblank;
 
             int seenSlash = 0;
             for(wstring::const_iterator it = tmpword[0]->begin(); it != tmpword[0]->end(); it++)
@@ -2122,6 +2412,47 @@ Transfer::transfer(FILE *in, FILE *out)
                 }
                 continue;
               }
+              else if(*it == L'[')
+              {
+                if(*(it+1) == L'[') //wordbound blank
+                {
+                  while(true)
+                  {
+                    if(*it == L'\\')
+                    {
+                      wblank.push_back(*it);
+                      it++;
+                      wblank.push_back(*it);
+                    }
+                    else if(*it == L'^' && *(it-1) == L']' && *(it-2) == L']')
+                    {
+                      break;
+                    }
+                    else
+                    {
+                      wblank.push_back(*it);
+                    }
+                    
+                    it++;
+                  }
+                }
+                else
+                {
+                  if(seenSlash == 0)
+                  {
+                    sl.push_back(*it);
+                  }
+                  else if(seenSlash == 1)
+                  {
+                    tl.push_back(*it);
+                  }
+                  else
+                  {
+                    ref.push_back(*it);
+                  }
+                }
+                continue;
+              }
               else if(*it == L'/')
               {
                 seenSlash++;
@@ -2144,6 +2475,7 @@ Transfer::transfer(FILE *in, FILE *out)
             }
             //tmpword[0]->assign(sl);
             tr = pair<wstring, int>(tl, false);
+            tr_wblank = wblank;
             //wcerr << L"pb: " << *tmpword[0] << L" :: " << sl << L" >> " << tl << endl ;
           }
           else
@@ -2155,23 +2487,31 @@ Transfer::transfer(FILE *in, FILE *out)
 	  {
 	    if(defaultAttrs == lu)
 	    {
-	      fputwc_unlocked(L'^', output);
+        if(tr.first[0] != L'[' || tr.first[1] != L'[')
+        {
+          fputws_unlocked(tr_wblank.c_str(), output);
+          fputwc_unlocked(L'^', output);
+        }
 	      fputws_unlocked(tr.first.c_str(), output);
 	      fputwc_unlocked(L'$', output);
-            }
-            else
-            {
-              if(tr.first[0] == '*')
-              {
-                fputws_unlocked(L"^unknown<unknown>{^", output);
-              }
-              else
-              {
-	        fputws_unlocked(L"^default<default>{^", output);
-              }
-	      fputws_unlocked(tr.first.c_str(), output);
-	      fputws_unlocked(L"$}$", output);
-            }
+      }
+      else
+      {
+        if(tr.first[0] == '*')
+        {
+          fputws_unlocked(L"^unknown<unknown>{", output);
+          fputws_unlocked(tr_wblank.c_str(), output);
+          fputwc_unlocked(L'^', output);
+        }
+        else
+        {
+          fputws_unlocked(L"^default<default>{", output);
+          fputws_unlocked(tr_wblank.c_str(), output);
+          fputwc_unlocked(L'^', output);
+        }
+          fputws_unlocked(tr.first.c_str(), output);
+          fputws_unlocked(L"$}$", output);
+      }
 	  }
 	  banned_rules.clear();
 	  tmpword.clear();
@@ -2202,6 +2542,7 @@ Transfer::transfer(FILE *in, FILE *out)
       lastrule = rule_map[val-1];
       lastrule_id = val;
       last = input_buffer.getPos();
+      last_lword = tmpword.size();
 
       if(trace)
       {
@@ -2266,30 +2607,25 @@ Transfer::applyRule()
       word = new TransferWord *[limit];
       std::fill(word, word+limit, (TransferWord *)(0));
       lword = limit;
-      if(limit != 1)
-      {
-        blank = new string *[limit - 1];
-        lblank = limit - 1;
-      }
-      else
-      {
-        blank = NULL;
-        lblank = 0;
-      }
     }
     else
     {
-      blank[i-1] = new string(UtfConverter::toUtf8(*tmpblank[i-1]));
+      if(int(blank_queue.size()) < last_lword - 1)
+      {
+        string blank_to_add = string(UtfConverter::toUtf8(*tmpblank[i-1]));
+        blank_queue.push(blank_to_add);
+      }
     }
 
     pair<wstring, int> tr;
     if(useBilingual && preBilingual == false)
     {
       tr = fstp.biltransWithQueue(*tmpword[i], false);
-      wstring refx;
+      wstring refx,wblankx;
       word[i] = new TransferWord(UtfConverter::toUtf8(*tmpword[i]),
                                  UtfConverter::toUtf8(tr.first),
                                  UtfConverter::toUtf8(refx),
+                                 UtfConverter::toUtf8(wblankx),
                                  tr.second);
     }
     else if(preBilingual)
@@ -2297,6 +2633,7 @@ Transfer::applyRule()
       wstring sl;
       wstring tl;
       wstring ref;
+      wstring wblank;
 
       int seenSlash = 0;
       for(wstring::const_iterator it = tmpword[i]->begin(); it != tmpword[i]->end(); it++)
@@ -2323,6 +2660,47 @@ Transfer::applyRule()
           }
           continue;
         }
+        else if(*it == L'[')
+        {
+          if(*(it+1) == L'[') //wordbound blank
+          {
+            while(true)
+            {
+              if(*it == L'\\')
+              {
+                wblank.push_back(*it);
+                it++;
+                wblank.push_back(*it);
+              }
+              else if(*it == L'^' && *(it-1) == L']' && *(it-2) == L']')
+              {
+                break;
+              }
+              else
+              {
+                wblank.push_back(*it);
+              }
+              
+              it++;
+            }
+          }
+          else
+          {
+            if(seenSlash == 0)
+            {
+              sl.push_back(*it);
+            }
+            else if(seenSlash == 1)
+            {
+              tl.push_back(*it);
+            }
+            else
+            {
+              ref.push_back(*it);
+            }
+          }
+          continue;
+        }
 
         if(*it == L'/')
         {
@@ -2348,15 +2726,17 @@ Transfer::applyRule()
       word[i] = new TransferWord(UtfConverter::toUtf8(sl),
                                  UtfConverter::toUtf8(tr.first),
                                  UtfConverter::toUtf8(ref),
+                                 UtfConverter::toUtf8(wblank),
                                  tr.second);
     }
     else // neither useBilingual nor preBilingual (sl==tl)
     {
       tr = pair<wstring, int>(*tmpword[i], false);
-      wstring refx;
+      wstring refx,wblankx;
       word[i] = new TransferWord(UtfConverter::toUtf8(*tmpword[i]),
                                  UtfConverter::toUtf8(tr.first),
-                                 UtfConverter::toUtf8(refx), 
+                                 UtfConverter::toUtf8(refx),
+                                 UtfConverter::toUtf8(wblankx),
                                  tr.second);
     }
   }
@@ -2373,17 +2753,7 @@ Transfer::applyRule()
     }
     delete[] word;
   }
-  if(blank)
-  {
-    for(unsigned int i = 0; i != limit - 1; i++)
-    {
-      delete blank[i];
-      blank[i] = 0;
-    }
-    delete[] blank;
-  }
   word = NULL;
-  blank = NULL;
   tmpword.clear();
   tmpblank.clear();
   ms.init(me->getInitial());
@@ -2395,6 +2765,7 @@ void
 Transfer::applyWord(wstring const &word_str)
 {
   ms.step(L'^');
+    
   for(unsigned int i = 0, limit = word_str.size(); i < limit; i++)
   {
     switch(word_str[i])
@@ -2403,7 +2774,33 @@ Transfer::applyWord(wstring const &word_str)
         i++;
 	ms.step(towlower(word_str[i]), any_char);
 	break;
-
+        
+      case L'[':
+        if(word_str[i+1] == L'[')
+        {
+          while(true)
+          {
+            if(word_str[i] == L'\\')
+            {
+              i++;
+            }
+            else if(i >= 4)
+            {
+              if(word_str[i] == L'^' && word_str[i-1] == L']' && word_str[i-2] == L']')
+              {
+                break;
+              }
+            }
+            
+            i++;
+          }
+        }
+        else
+        {
+          ms.step(towlower(word_str[i]), any_char);
+        }
+        break;
+        
       case L'/':
         i = limit;
         break;
