@@ -8,7 +8,9 @@ import unittest
 import tagger
 import pretransfer
 import transfer
+import interchunk
 import postchunk
+import adaptdocx
 
 if __name__ == "__main__":
     os.chdir(os.path.dirname(__file__))
@@ -16,8 +18,11 @@ if __name__ == "__main__":
     for module in [tagger,
                    pretransfer,
                    transfer,
-                   postchunk]:
+                   interchunk,
+                   postchunk,
+                   adaptdocx]:
         suite = unittest.TestLoader().loadTestsFromModule(module)
-        res = unittest.TextTestRunner(verbosity = 2).run(suite)
-        failures += len(res.failures)
+        res = unittest.TextTestRunner(verbosity=2).run(suite)
+        if(not(res.wasSuccessful())):
+            failures += 1
     sys.exit(min(failures, 255))
